@@ -78,182 +78,6 @@
 (defmacro in-type () 2)
 (defmacro out-type () 3)
 
-(defxdoc aignet
-  :parents (acl2::boolean-reasoning)
-  :short "An efficient, @(see acl2::stobj)-based And-Inverter Graph (AIG)
-representation for Boolean functions and finite-state machines."
-  :long
-  "<p>An and-inverter graph (AIG) at its most basic is a DAG whose nodes are either
-AND gates, outputs, or inputs.  Outputs have 1 descendant, ANDs have 2, and
-inputs have none.  An edge may point to an AND or an input, but not an output.
-Each edge has a Boolean attribute signifying whether it is negated or not.</p>
-
-<p>We call this a combinational AIG.  AIGNET, following packages like ABC,
-implements a sequential AIG, by dividing the inputs and outputs into two
-categories, \"register\" and \"primary\".  A sequential AIG can be seen as a
-combinational AIG by just ignoring these distinctions; when we wish to ignore
-the register/primary distinctions we refer to <i> combinational inputs </i>
-and <i> combinational outputs </i>.  Confusingly, combinational inputs of
-the register type are called <i>register outputs</i> -- it is an output
-from a register, but are read (like inputs) by the combinational logic -- and
-combinational outputs of the register type are called <i>register
-inputs</i> -- they are inputs to registers, but are written (like outputs)
-by the combinational logic.  Register inputs, like primary outputs, have one
-descendant and may not be the descendant of another node; register outputs,
-like primary inputs, have no descendants and may be the descendant of another
-node.</p>
-
-<p>Sequential semantics for an AIG network can be summarized as follows:<br/>
- Initially, assign initial values to the register output nodes.<br/>
- Every cycle,<br/>
-  <ul><li>assign values to the primary inputs</li>
-      <li>copy values from register outputs to corresponding register inputs</li>
-      <li>compute values of AND gates in topological order</li>
-      <li>compute values of primary outputs and register inputs.</li></ul></p>
-
-<p>During the period of computing values for ANDs/primary outputs/register inputs,
-the circuit is basically treated as combinational -- register outputs and
-primary inputs are treated identically, as are primary outputs and register
-inputs.</p>
-
-<p>Usage and implementation are discussed in the subtopics.</p>")
-
-(defsection aignet-impl
-  :parents (aignet)
-  :short "Implementation details of AIGNET"
-  :long
-  "<p>The AIGNET network consists mainly of an array of nodes representing a
-topologically sorted DAG described in @(see aignet).  This uses a stobj called
-@('AIGNET').</p>
-
-<p>Each node in the graph has an ID, a natural number that can be used to look
-up that node in the array.  However, often our functions take arguments that
-may be a node or its negation; we encode this as a <i>literal</i>, as 2*ID+NEG,
-where NEG is 1 signifying negated or 0 signifying not negated.</p>
-
-<p>One arbitrary well-formedness condition that we impose on all AIGNET
-networks is that it must have a unique constant-false node with index 0.
-Therefore, the literal 0 is constant-false (the constant-false node, not
-negated), and the literal 1 is constant-true (the constant-false node,
-negated).</p>
-
-<p>Information about each node is stored in the node array as two consecutive
-32-bit numbers, and the node ID corresponds to its position: the two array
-indices of a node are 2*ID and 2*ID+1.  The two 32-bit values contained at
-these locations are broken down into two 30-bit data slots and four extra bits.
-Three of the four extra bits encode the type of the node, and the last extra
-bit encodes the phase of the node, which is its value when all inputs/registers
-are 0.  The meaning of the 30-bit data slots depends on the type of node; in
-some cases it stores a literal.</p>
-
-<p>The encoding is broken down, least-significant bits first, as:</p>
-<ul>
-<li>2 bits:   combinational type</li>
-<li>30 bits:  data slot 0</li>
-<li>1 bit:    register flag</li>
-<li>1 bit:    phase</li>
-<li>30 bits:  data slot 1.</li></ul>
-
-<p>The combinational types are:</p>
-<ul>
-<li>0:   constant false</li>
-<li>1:   gate</li>
-<li>2:   input</li>
-<li>3:   output</li></ul>
-
-<p>The register flag only applies to combinational inputs/outputs; it should be
-0 for constants/gates (but should also be ignored in those cases).  An input
-with the register flag set is a register output, and an output with the
-register flag set is a register input.  (Remember that the output of a register
-is an input to the combinational logic, and the input to a register is an
-output from the combinational logic.)</p>
-
-<p>So there are, in total, six types of object, encoded as follows:
-<code>
-Name               Type encoding          Register flag
-Constant                 0                    -
-AND gate                 1                    -
-Primary input            2                    0
-Register output          2                    1
-Primary output           3                    0
-Register input           3                    1.
-</code></p>
-
-<p>Only objects with type 0, 1, 2 -- constants, gates, and combinational inputs
--- may be fanins (descendents) of AND or combinational output objects;
-combinational outputs (type 3) may not.</p>
-
-<p>The meanings of the two 30-bit data slots vary based on the type:</p>
-
-<p>Constant: both data 0 and 1 are meaningless, and should be set to 0 but
-ignored</p>
-
-<p>AND gates: data 0 and 1 are literals encoding the fanins to the gate.  The
-ID part of each literal must be less than the gate ID.</p>
-
-<p>Combinational inputs: data 0 is ignored and should be 0, and fanin 1 gives
-the PI or register number, sequentially assigned and unique among
-PI/registers.</p>
-
-<p>Primary outputs: data 0 is the fanin (literal) of the output, whose ID must
-be less than the output node ID.  Data 1 is the output number.</p>
-
-<p>Register inputs: data 0 is the fanin (literal) of the register, whose ID
-must be less than this node's ID.  Fanin 1 is the ID of the corresponding
-register output, which must also be less than this node's ID.  (The register
-number of the RI is the register number of the corresponding RO.)</p>
-
-<p>Having separate register input and output objects is somewhat awkward in
-terms of saying when a network is well-formed.  In some sense, it's not
-well-formed unless every RO has a corresponding RI.  But the RIs can't be
-added until their input cones are built, and we'd like to be able to say
-the network is well-formed in some sense while it is being built.  So when
-an RO has no corresponding RI, we will say it is simply not updated -- its
-value under sequential evalutation remains the same at each frame.  A
-separate predicate can check that this isn't the case, but we won't
-generally require this for guards etc.  Furthermore, for convenience we
-also allow RIs with fanin 1 set to 0 -- in this case, they are not proper
-RIs because they do not connect to an RO, and they have no register
-number.  They are also basically irrelevant to any sequential computation,
-because no RI gets updated with their previous value.</p>
-
-<p>We require that each RI object occur later (have a larger ID) than its
-corresponding RO object.  This allows a couple of conveniences:</p>
-<ul>
-<li>there is a function for adding an RO and another function for adding an
-  RI which connects it to an existing RO.  If we allowed RIs to occur first,
-  then we'd need an additional pair of functions, for adding an unconnected
-  RI and for adding an RO connected to an RI.  Maybe we could avoid this by
-  separately allowing RO/RIs to be connected, but this seems knotty in terms
-  of well-formednes.</li>
-<li>When doing a sequential simulation or similar operation that involves
-  repeated sweeps across the AIG nodes, an RO node will be reached before
-  the corresponding RI's previous value is overwritten.  So we don't need an
-  addtional step of copying RI-&gt;RO values between frames.</li></ul>
-<p>Also, a strategy that might alleviate any inconvenience due to needing to
-add the RO before the RI: at the point of adding the RI, check whether the
-RO already exists and add it first if not.</p>
-
-<p>An AIGNET network is designed to have objects added one at a time without
-later modification.  That is, new objects may be added, but existing
-objects are not changed.  The object array itself (along with its size)
-contains enough information to fully replicate the network; in this sense,
-all other information recorded is auxiliary.  But we do also keep arrays
-of inputs, outputs, and registers.  The input and output arrays simply
-hold the IDs of inputs/outputs in the order that they were added (as
-described below, each input/output object records its input/output
-numbers, i.e. the index in the input/output array that points back to
-it).  The register array is a bit more complicated, because there are
-typically two nodes (register input and register output) associated with
-each register.  Each entry in the register array contains the ID of either
-a register input or output.  If it is a register input, then the register
-is incomplete, i.e. its output hasn't been added yet.  If it is a register
-output, then we have a complete register: the register array points to the
-register output node, which points to the register input node, which has
-the index in the register array.</p>")
-
-
-
 ;; (defsection aignet-untranslate
 ;;   (defun untranslate-preproc-node-types (term wrld)
 ;;     (declare (ignore wrld))
