@@ -19,26 +19,18 @@
 ; Original author: Jared Davis <jared@centtech.com>
 
 (in-package "ACL2")
-
-; ihs-extensions.lisp
-;
-; This book is can be used instead of ihs/logops-lemmas to add a bunch of
-; additional theorems and provide a more sensible theory.  This book should
-; typically be only locally included since it makes various changes to the
-; global theory such as disabling floor, mod, expt, etc.
-
 (include-book "ihsext-basics")
-;; (include-book "defaults")
 (include-book "integer-length")
-
-; (local (include-book "arithmetic-3/extra/top-ext" :dir :system))
 (local (include-book "equal-by-logbitp"))
-
-; (local (in-theory (disable expt-between-one-and-two)))
-
-
 (local (in-theory (enable* arith-equiv-forwarding)))
 
+(defsection bitops/ihs-extensions
+  :parents (bitops)
+  :short "Extension of @(see bitops/ihsext-basics) with additional lemmas."
+
+  :long "<p>BOZO this needs a lot of documentation.  For now you're best
+off looking at the source code.</p>")
+
 
 (local (defun dec-induct (n)
          (if (zp n)
@@ -452,7 +444,7 @@
                           (ash-1-removal
                            exponents-add-for-nonneg-exponents
                            expt))))
-  
+
   (defthm |(2^n + a mod 2^n) when a < 2^n+1|
     (implies (and (case-split (< a (expt 2 (+ 1 n))))
                   (natp a)
@@ -566,7 +558,11 @@
            :hints (("goal" :use lemma2
                     :in-theory (disable lemma2)))))
 
-  (defthmd |(integerp (expt x n))|
+  (defthmd integerp-of-expt
+    ;; Previously this was named |(integerp (expt x n))|, but that had a name
+    ;; conflict with arithmetic-5.  Unfortunately the rules differ---this rule
+    ;; is better than arithmetic 5's---so for now I'll just rename this one to
+    ;; avoid the conflict.
     (implies (and (integerp x)
                   (< 1 x))
              (equal (integerp (expt x n))
@@ -574,7 +570,7 @@
     :hints(("Goal" :in-theory (enable expt)))))
 
 
-(local (in-theory (enable |(integerp (expt x n))|)))
+(local (in-theory (enable integerp-of-expt)))
 
 (defthmd |(logcdr (expt 2 n))|
   (equal (logcdr (expt 2 n))
@@ -583,8 +579,8 @@
            0))
   :hints(("Goal" :in-theory (e/d* (ihsext-arithmetic)
                                   (ash-1-removal
-                                   |(integerp (expt x n))|))
-          :use ((:instance |(integerp (expt x n))|
+                                   integerp-of-expt))
+          :use ((:instance integerp-of-expt
                  (x 2))))
          '(:cases ((zip n)))))
 
@@ -597,8 +593,8 @@
            0))
   :hints(("Goal" :in-theory (e/d* (ihsext-arithmetic)
                                   (ash-1-removal
-                                   |(integerp (expt x n))|))
-          :use ((:instance |(integerp (expt x n))|
+                                   integerp-of-expt))
+          :use ((:instance integerp-of-expt
                  (x 2)))
           :cases ((natp n)))))
 
@@ -789,7 +785,7 @@
 
 
 (def-ruleset ihs-ext-disabled-rules
-  '(|(integerp (expt x n))|
+  '(integerp-of-expt
     |(logcdr (expt 2 n))|
     |(logcar (expt 2 n))|
     logand-with-2^n-is-logbitp
@@ -798,11 +794,14 @@
     |(logbitp n (+ (expt 2 n) a))|))
 
 
-(encapsulate 
+(encapsulate
  ()
- (local (include-book "arithmetic/top-with-meta" :dir :system))            
+ (local (include-book "arithmetic/top-with-meta" :dir :system))
 
- (defthm lognot-lognot
+ (defthm lognot-of-lognot
+   ;; Renamed from lognot-lognot to lognot-of-lognot because arithmetic-5
+   ;; and ihs/logops-lemmas both have a worse version of this rule named
+   ;; lognot-lognot.
    (equal (lognot (lognot x))
           (ifix x))
    :hints (("Goal" :in-theory (enable lognot)))
@@ -813,14 +812,14 @@
     (unsigned-byte-p n x)
     (equal (loghead n (- (loghead n (- x))))
            x))
-   :hints (("Goal" 
+   :hints (("Goal"
             :induct t
             :in-theory (e/d* (minus-to-lognot
                               ihsext-recursive-redefs
                               ihsext-inductions))
             :expand ((:free (x) (loghead n x))))))
 
- 
+
  (defthm loghead-of-negative-rw
    (implies
     (and
