@@ -20,7 +20,7 @@
 
 (in-package "CLEX")
 (include-book "top")
-(include-book "cutil/top" :dir :system)
+(include-book "std/util/top" :dir :system)
 (include-book "str/top" :dir :system)
 
 (local (defthm characterp-of-car-when-character-listp
@@ -459,4 +459,4 @@ calls @(see lex-main) to do the real work.</p>"
 
 (lex-string "a + b + #3 + 4" :filename "foo.c")
 
-||#
\ No newline at end of file
+||#
