@@ -101,8 +101,8 @@ returns is also @('nil').</p>"
     (declare (xargs :mode :program))
     (b* ((__function__ 'def-match-thms)
          (name-s  (symbol-name name))
-         (formals (cutil::look-up-formals name world))
-         (returns (cutil::look-up-return-vals name world))
+         (formals (std::look-up-formals name world))
+         (returns (std::look-up-return-vals name world))
          ((unless (member 'sin formals))
           (raise "~x0 does not take ~x1 as an argument." name 'sin))
          ((unless (or (equal returns '(nil sin))
