@@ -21,17 +21,121 @@
 (in-package "RSTOBJ")
 (include-book "typed-records")
 
-; def-typed-record.lisp
-;
-; We introduce the DEF-TYPED-RECORD macro which can be used to introduce a new
-; kind of typed record.  Using this macro gives you definitions of the get and
-; set functions, the ordinary get-of-set style theorems about them, and other
-; stuff such as a badguy for finding differences between typed records.
-;
-; The generic theory that supports this macro is found in typed-records.lisp.
-;
-; For several examples of using the macro, see typed-record-tests.lisp.
+(defsection def-typed-record
+  :parents (defrstobj)
+  :short "Introduce a typed record for use with @(see defrstobj)."
 
+  :long "<p>A <b>typed record</b> is a record-like structure: it associates
+keys with values, has a @('get') function to look up the value of some key, and
+has a @('set') function to install some new value for some key.</p>
+
+<p>Unlike an ordinary @('misc/record'), a typed record is homogeneous, i.e., we
+unconditionally know @({(foop (get a r))}).  Meanwhile, the @('get') and
+@('set') functions for a typed record are <b>almost</b> the same as for
+ordinary records.  The only difference is that the @('g-same-s') theorem
+becomes:</p>
+
+@({
+   (get a (set a v r)) = (foo-fix v)   ; instead of just being v
+})
+
+<p>The macro @('def-typed-record') can be used to introduce a new typed record
+structure for use in @(see defrstobj) array fields.</p>
+
+
+<h3>Usage</h3>
+
+<p>You can use @('def-typed-record') to introduce the @('get-') and @('set-')
+functions, the ordinary @('get-of-set') style theorems about them, and some
+additional definitions such as a badguy for identifying differences between
+typed records (which can be useful for pick-a-point style reasoning.)</p>
+
+<h5>Example: typed record for naturals</h5>
+
+@({
+    (def-typed-record nat
+      :elem-p (natp x)
+      :elem-list-p (nat-listp x)
+      :elem-default 0
+      :elem-fix (nfix x))
+})
+
+<p>This introduces the recognizer function @('nat-tr-p'), the getter function
+@('nat-tr-get'), the setter function @('nat-tr-set'), and related theorems.</p>
+
+<h5>Example: typed-record for (8-bit) bytes</h5>
+
+@({
+   (defun ubp-listp (n x)
+     (declare (xargs :guard t))
+     (if (atom x)
+         (not x)
+       (and (unsigned-byte-p n (car x))
+            (ubp-listp n (cdr x)))))
+
+   (defun ubp-fix (n x)            ;; other fixing functions
+     (declare (xargs :guard t))    ;; are also possible
+     (if (unsigned-byte-p n x)
+         x
+       0))
+
+   (def-typed-record ubp8
+     :elem-p       (unsigned-byte-p 8 x)
+     :elem-list-p  (ubp-listp 8 x)
+     :elem-default 0
+     :elem-fix     (ubp-fix 8 x))
+})
+
+<p>This produce @('ubp8-tr-p'), @('ubp8-tr-get'), @('ubp8-tr-set'), and related
+theorems.</p>
+
+
+<h5>General Form</h5>
+
+@({
+    (def-typed-record name
+      :elem-p        [element recognizer, foop]
+      :elem-list-p   [list recognizer, foo-listp]
+      :elem-default  [default value, e.g., 0, nil, ...]
+      :elem-fix      [fixing function, foo-fix]
+      :in-package-of [symbol to use for new names])
+})
+
+<p>Note that the terms you use for @('elem-p') and so forth need to refer to
+exactly the variable @('acl2::x').</p>
+
+
+<h3>Related Work and History</h3>
+
+<p>This version of typed records is very similar in spirit to the
+@('coi/records/defrecord.lisp') book; see:</p>
+
+<box><p>David Greve and Matthew Wilding.  <a
+href='http://www.cs.utexas.edu/users/moore/acl2/workshop-2003/contrib/greve-wilding_defrecord/defrecord.ps'>Typed
+ACL2 Records</a>. ACL2 Workshop 2003.</p></box>
+
+<p>Greve and Wilding implemented typed records by starting with an ordinary
+record, but instead of just storing values or fixed values into its slots, they
+instead store ENTRIES of the form @('(FOO . NON-ENTRY)'), where the FOO must be
+a foop that is not the default foop.</p>
+
+<p>When developing rstobjs, I (Jared) started with Greve and Wilding's
+approach; see @(see legacy-defrstobj).  But when I tried to extend their work
+to be able to view a STOBJ array as a typed record, I ran into trouble.  I
+didn't see a good way to prove something akin to @('equal-by-g'), and without
+that it didn't seem easy to adapt the basic approach I'd developed for untyped
+records to also deal with typed records.</p>
+
+<p>I went to Sol's office to gripe about this, and we started to try to more
+precisely understand what was problematic.  In a short time, we had come up
+with a different way to implement typed records that seems to be much more
+suitable.</p>
+
+<p>In short, instead of using a @('misc/record') with some kind of special
+entries, we directly develop a new kind of typed record where the well-formed
+records are only allowed to have values of the proper type.  This lets us
+nicely separate the bad part of the record (if any) from the good part, and
+obtain a theorem in the spirit of @('EQUAL-BY-G').</p>")
 
 (defun symbol-list-names (x)
   (declare (xargs :guard (symbol-listp x)))
@@ -61,7 +165,6 @@
 
    elem-fix      ; A term mentioning only X, the fixing function
                  ;  e.g., (nfix x), (ubp-fix 8 x), etc.
-                 ;  must be equal to (if elem-p x elem-default)
 
    in-package-of ; Package for names we'll generate
 
@@ -102,7 +205,8 @@
 
          (booleanp-of-elem-p    (mksym 'booleanp-of-elem-p-for- tr-p))
          (elem-p-of-default     (mksym 'elem-p-of-default-for- tr-p))
-         (elem-fix-correct      (mksym 'elem-fix-correct-for- tr-p))
+         (elem-p-of-elem-fix    (mksym 'elem-p-of-elem-fix-for- tr-p))
+         (elem-fix-idempotent   (mksym 'elem-fix-idempotent-for- tr-p))
          (elem-list-p-when-atom (mksym 'elem-list-p-when-atom-for- tr-p))
          (elem-list-p-of-cons   (mksym 'elem-list-p-of-cons-for- tr-p)))
 
@@ -124,11 +228,13 @@
          ;; to write (symbolp nil) as a rewrite rule, or similar.
          :rule-classes :built-in-clause)
 
-       (defthmd ,elem-fix-correct
-         (equal ,elem-fix
-                (if ,elem-p
-                    x
-                  ,elem-default)))
+       (defthmd ,elem-p-of-elem-fix
+         (let ((x ,elem-fix))
+           ,elem-p))
+
+       (defthmd ,elem-fix-idempotent
+         (implies ,elem-p
+                  (equal ,elem-fix x)))
 
        (defthmd ,elem-list-p-when-atom
          (implies (atom x)
@@ -155,7 +261,8 @@
                                  booleanp
                                  ,booleanp-of-elem-p
                                  ,elem-p-of-default
-                                 ,elem-fix-correct
+                                 ,elem-fix-idempotent
+                                 ,elem-p-of-elem-fix
                                  ,elem-list-p-when-atom
                                  ,elem-list-p-of-cons)))
 
@@ -234,27 +341,30 @@
          (declare (xargs :guard (and (natp n)
                                      (true-listp arr))))
          (if (zp n)
-             (,tr-set 0 (nth 0 arr) rec)
-           (,array-to-tr (- n 1) arr (,tr-set n (nth n arr) rec))))
+             rec
+           (let ((n (- n 1)))
+             (,array-to-tr n arr (,tr-set n (nth n arr) rec)))))
 
        (defun ,tr-to-array (n rec arr)
          (declare (xargs :guard (and (natp n)
                                      (true-listp arr))))
          (if (zp n)
-             (update-nth 0 (,tr-get 0 rec) arr)
-           (,tr-to-array (- n 1) rec (update-nth n (,tr-get n rec) arr))))
+             arr
+           (let ((n (- n 1)))
+             (,tr-to-array n rec (update-nth n (,tr-get n rec) arr)))))
 
        (defun ,tr-delete-indices (n rec)
          (declare (xargs :guard (natp n)))
          (if (zp n)
-             (,tr-set 0 ,elem-default rec)
-           (,tr-delete-indices (- n 1) (,tr-set n ,elem-default rec))))
+             rec
+           (let ((n (- n 1)))
+             (,tr-delete-indices n (,tr-set n ,elem-default rec)))))
 
        (defun ,array-rec-pair-p (arr rec len)
-         (declare (xargs :guard (posp len)))
+         (declare (xargs :guard (natp len)))
          (and ,(subst 'arr 'x elem-list-p)
               (= (len arr) len)
-              (equal rec (,tr-delete-indices (- len 1) rec))))
+              (equal rec (,tr-delete-indices len rec))))
 
 
        (deflabel ,(mksym 'start-of- tr-p '-theorems))
@@ -393,7 +503,7 @@
        (defthm ,(mksym tr-get '-of- array-to-tr)
          (equal (,tr-get key (,array-to-tr n arr rec))
                 (if (and (natp key)
-                         (<= key (nfix n)))
+                         (< key (nfix n)))
                     ,(subst '(nth key arr) 'x elem-fix)
                   (,tr-get key rec)))
          :hints(("Goal" :use ((:functional-instance tr-get-of-array-to-tr
@@ -402,19 +512,19 @@
 
        (defthm ,(mksym 'len-of- tr-to-array)
          (equal (len (,tr-to-array n rec arr))
-                (max (+ 1 (nfix n)) (len arr)))
+                (max (nfix n) (len arr)))
          :hints(("Goal" :use ((:functional-instance len-of-tr-to-array
                                                     . ,fi-pairs)))))
 
        (defthm ,(mksym 'elem-list-p-of- tr-to-array)
          (implies (and ,(subst 'arr 'x elem-list-p)
-                       (< (nfix n) (len arr)))
+                       (<= (nfix n) (len arr)))
                   ,(subst `(,tr-to-array n rec arr) 'x elem-list-p))
          :hints(("Goal" :use ((:functional-instance elem-list-p-of-tr-to-array
                                                     . ,fi-pairs)))))
 
        (defthm ,(mksym tr-to-array '-idempotent)
-         (implies (and (force (posp (len arr1)))
+         (implies (and (force (natp (len arr1)))
                        (force ,(subst 'arr1 'x elem-list-p)))
                   (equal (,tr-to-array n rec1 (,tr-to-array n rec2 arr1))
                          (,tr-to-array n rec1 arr1)))
@@ -424,7 +534,7 @@
        (defthm ,(mksym tr-to-array '-of- tr-set)
          (implies (and (natp n)
                        (natp i)
-                       (<= i n)
+                       (< i n)
                        ,(subst 'val 'x elem-p)
                        ,(subst 'arr 'x elem-list-p))
                   (equal (,tr-to-array n (,tr-set i val rec) arr)
@@ -434,7 +544,7 @@
 
        (defthm ,(mksym tr-to-array '-of- array-to-tr)
          (implies (and (force (equal (len arr1) (len arr2)))
-                       (force (equal n (- (len arr1) 1)))
+                       (force (equal n (len arr1)))
                        (force (posp (len arr1)))
                        (force ,(subst 'arr1 'x elem-list-p))
                        (force ,(subst 'arr2 'x elem-list-p)))
@@ -461,7 +571,7 @@
        (defthm ,(mksym tr-delete-indices '-of- tr-set)
          (implies (and (natp n)
                        (natp i)
-                       (<= i n))
+                       (< i n))
                   (equal (,tr-delete-indices n (,tr-set i val rec))
                          (,tr-delete-indices n rec)))
          :hints(("Goal" :use ((:functional-instance tr-delete-indices-of-tr-set
@@ -485,8 +595,8 @@
        (defthmd ,(mksym 'equal-of- array-to-tr)
          (implies (and (,array-rec-pair-p arr1 rec1 len1)
                        (,array-rec-pair-p arr2 rec2 len2)
-                       (equal len1 (+ 1 len))
-                       (equal len2 (+ 1 len))
+                       (equal len1 len)
+                       (equal len2 len)
                        (natp len))
                   (equal (equal (,array-to-tr len arr1 rec1)
                                 (,array-to-tr len arr2 rec2))
@@ -505,7 +615,8 @@
                           (union-theories
                            '(,booleanp-of-elem-p
                              ,elem-p-of-default
-                             ,elem-fix-correct
+                             ,elem-fix-idempotent
+                             ,elem-p-of-elem-fix
                              ,elem-list-p-when-atom
                              ,elem-list-p-of-cons)
                            (set-difference-theories
@@ -547,7 +658,7 @@
 
 ; Some basic tests to make sure the macro is working
 
-   (def-typed-record nat
+(def-typed-record nat
      :elem-p (natp x)
      :elem-list-p (nat-listp x)
      :elem-default 0
@@ -573,4 +684,19 @@
      :elem-default 0
      :elem-fix     (ubp-fix 8 x))
 
+
+   (defun nonneg-fix (x)
+     (declare (xargs :guard t))
+     (if (integerp x)
+         (if (< x 0)
+             (- x)
+           x)
+       0))
+
+   (def-typed-record nonneg
+     :elem-p (natp x)
+     :elem-list-p (nat-listp x)
+     :elem-default 0
+     :elem-fix (nonneg-fix x))
+
    ))
