@@ -23,44 +23,9 @@
 (include-book "std/lists/mfc-utils" :dir :system)
 (include-book "centaur/misc/introduce-var" :dir :system)
 (local (include-book "std/lists/nth" :dir :system))
-(local (include-book "array-lemmas"))
 (set-verify-guards-eagerness 2)
 
 
-; typed-records.lisp
-;
-; This file is similar in spirit to coi/records/defrecord.lisp, which was
-; described in the following paper.
-;
-;   David Greve and Matthew Wilding.  Typed ACL2 Records. ACL2 Workshop 2003.
-;
-; Loosely speaking, our goal here is to develop a record-like structure where
-; the values always have a particular type.  The resulting structure satisfies
-; almost the same theorems as in the misc/records book, but the g-same-s
-; theorem becomes
-;
-;   (g a (s a v r)) = (foo-fix v)      ; instead of just being v
-;
-; And we gain the theorem (foop (g a r)).
-;
-; Greve and Wilding accomplish this by starting with an ordinary record, but
-; instead of just storing values or fixed values into its slots, they instead
-; store ENTRIES of the form (FOO . NON-ENTRY), where the FOO must be a foop
-; that is not the default foop.  I started with this, but when I tried to
-; extend it to be able to view a STOBJ array as a typed record, I ran into
-; trouble.  I didn't see a good way to prove something akin to EQUAL-BY-G, and
-; without that it didn't seem easy to adapt the basic approach I'd developed
-; for untyped records to typed records.
-;
-; I went to Sol's office to gripe about this, and we started to try to more
-; precisely understand what was problematic.  In a short time, we had come up
-; with a different way to implement typed records that seems to be much more
-; suitable.  In short, instead of using a misc/record with some kind of special
-; entries, we directly develop a new kind of typed record where the well-formed
-; records are only allowed to have values of the proper type.  This lets us
-; nicely separate the bad part of the record (if any) from the good part, and
-; obtain a theorem in the spirit of EQUAL-BY-G.
-;
 ; This book develops a generic theory for typed records.  By itself this
 ; generic theory is not very useful, so you will want to also be aware of:
 ;
@@ -82,23 +47,32 @@
 
 (encapsulate
   (((elem-p *) => *)
-   ((elem-default) => *))
+   ((elem-default) => *)
+   ((elem-fix *) => *))
 
   (local (defun elem-p (x) (natp x)))
   (local (defun elem-default () 0))
+  (local (defun elem-fix (x) (if (natp x) x 0)))
 
   (defthm booleanp-of-elem-p
     (booleanp (elem-p x))
     :rule-classes :type-prescription)
 
   (defthm elem-p-of-elem-default
-    (elem-p (elem-default))))
+    (elem-p (elem-default)))
 
-(defun elem-fix (x)
-  ;; Standard fixing function.
-  (if (elem-p x)
-      x
-    (elem-default)))
+  (defthm elem-p-of-elem-fix
+    (elem-p (elem-fix x)))
+
+  (defthm elem-fix-idempotent
+    (implies (elem-p x)
+             (equal (elem-fix x) x))))
+
+;; (defun elem-fix (x)
+;;   ;; Standard fixing function.
+;;   (if (elem-p x)
+;;       x
+;;     (elem-default)))
 
 (defun tr-p1 (x)
   ;; Main typed-record predicate (ordered alist where every key is bound
@@ -704,38 +678,41 @@
   (declare (xargs :guard (and (natp n)
                               (true-listp arr))))
   (if (zp n)
-      (tr-set 0 (nth 0 arr) rec)
-    (array-to-tr (- n 1) arr (tr-set n (nth n arr) rec))))
+      rec
+    (let ((n (- n 1)))
+      (array-to-tr n arr (tr-set n (nth n arr) rec)))))
 
 (defun tr-to-array (n rec arr)
   ;; Load arr[0]...arr[n] from rec[0]...rec[n]
   (declare (xargs :guard (and (natp n)
                               (true-listp arr))))
   (if (zp n)
-      (update-nth 0 (tr-get 0 rec) arr)
-    (tr-to-array (- n 1) rec (update-nth n (tr-get n rec) arr))))
+      arr
+    (let ((n (- n 1)))
+      (tr-to-array n rec (update-nth n (tr-get n rec) arr)))))
 
 (defun tr-delete-indices (n rec)
   ;; Delete rec[0]...rec[n] from rec
   (declare (xargs :guard (natp n)))
   (if (zp n)
-      (tr-set 0 (elem-default) rec)
-    (tr-delete-indices (- n 1) (tr-set n (elem-default) rec))))
+      rec
+    (let ((n (- n 1)))
+      (tr-delete-indices n (tr-set n (elem-default) rec)))))
 
 (defun array-rec-pair-p (arr rec len)
   ;; Recognize array/record pairs where the array has size LEN and the record
   ;; has nothing in keys 0...LEN-1.
-  (declare (xargs :guard (posp len)))
+  (declare (xargs :guard (natp len)))
   (and (elem-list-p arr)
        (= (len arr) len)
-       (equal rec (tr-delete-indices (- len 1) rec))))
+       (equal rec (tr-delete-indices len rec))))
 
 
 
 (defthm tr-get-of-array-to-tr
   (equal (tr-get key (array-to-tr n arr rec))
          (if (and (natp key)
-                  (<= key (nfix n)))
+                  (< key (nfix n)))
              (elem-fix (nth key arr))
            (tr-get key rec))))
 
@@ -750,28 +727,26 @@
 
 (defthm len-of-tr-to-array
   (equal (len (tr-to-array n rec arr))
-         (max (+ 1 (nfix n)) (len arr))))
+         (max  (nfix n) (len arr))))
 
 (defthm elem-list-p-of-tr-to-array
   (implies (and (elem-list-p arr)
-                (< (nfix n) (len arr)))
+                (<= (nfix n) (len arr)))
            (elem-list-p (tr-to-array n rec arr))))
 
 
 (defthm nth-of-tr-to-array
   (equal (nth key (tr-to-array n rec arr))
-         (cond ((zp key)
-                (tr-get 0 rec))
-               ((<= key (nfix n))
-                (tr-get key rec))
+         (cond ((< (nfix key) (nfix n))
+                (tr-get (nfix key) rec))
                (t
                 (nth key arr)))))
 
 (defthm nth-of-tr-to-array-of-array-to-tr
   (implies (and (natp key)
                 (natp n)
-                (<= key n)
-                (< n (len arr1))
+                (< key n)
+                (<= n (len arr1))
                 (equal (len arr1) (len arr2))
                 (elem-list-p arr1))
            (equal (nth key (tr-to-array n (array-to-tr n arr1 rec) arr2))
@@ -779,8 +754,8 @@
 
 (defthm tr-to-array-of-array-to-tr
   (implies (and (force (equal (len arr1) (len arr2)))
-                (force (equal n (- (len arr1) 1)))
-                (force (posp (len arr1)))
+                (force (equal n (len arr1)))
+                (force (natp (len arr1)))
                 (force (elem-list-p arr1))
                 (force (elem-list-p arr2)))
            (equal (tr-to-array n (array-to-tr n arr1 rec) arr2)
@@ -790,7 +765,7 @@
                  equal-by-nths
                  (equal-by-nths-hyp (lambda ()
                                       (and (equal (len arr1) (len arr2))
-                                           (equal n (- (len arr1) 1))
+                                           (equal n (len arr1))
                                            (elem-list-p arr1)
                                            (elem-list-p arr2))))
                  (equal-by-nths-lhs (lambda ()
@@ -799,7 +774,7 @@
                                       arr1)))))))
 
 (defthm tr-to-array-idempotent
-  (implies (and (force (posp (len arr1)))
+  (implies (and (force (natp (len arr1)))
                 (force (elem-list-p arr1)))
            (equal (tr-to-array n rec1 (tr-to-array n rec2 arr1))
                   (tr-to-array n rec1 arr1)))
@@ -807,7 +782,7 @@
           :use ((:functional-instance
                  equal-by-nths
                  (equal-by-nths-hyp (lambda ()
-                                      (and (posp (len arr1))
+                                      (and (natp (len arr1))
                                            (elem-list-p arr1))))
                  (equal-by-nths-lhs (lambda ()
                                       (tr-to-array n rec1 (tr-to-array n rec2 arr1))))
@@ -817,7 +792,7 @@
 (defthm tr-to-array-of-tr-set
   (implies (and (natp n)
                 (natp i)
-                (<= i n)
+                (< i n)
                 (elem-p val)
                 (elem-list-p arr))
            (equal (tr-to-array n (tr-set i val rec) arr)
@@ -828,7 +803,7 @@
                  (equal-by-nths-hyp (lambda ()
                                       (and (natp n)
                                            (natp i)
-                                           (<= i n)
+                                           (< i n)
                                            (elem-p val)
                                            (elem-list-p arr))))
                  (equal-by-nths-lhs (lambda ()
@@ -852,7 +827,7 @@
 (defthm tr-get-of-tr-delete-indices
   (equal (tr-get key (tr-delete-indices n rec))
          (if (and (natp key)
-                  (<= key (nfix n)))
+                  (< key (nfix n)))
              (elem-default)
            (tr-get key rec))))
 
@@ -871,7 +846,7 @@
 (defthm tr-delete-indices-of-tr-set
   (implies (and (natp n)
                 (natp i)
-                (<= i n))
+                (< i n))
            (equal (tr-delete-indices n (tr-set i val rec))
                   (tr-delete-indices n rec)))
   :hints(("Goal"
@@ -880,7 +855,7 @@
                  (equal-by-tr-get-hyp (lambda ()
                                    (and (natp n)
                                         (natp i)
-                                        (<= i n))))
+                                        (< i n))))
                  (equal-by-tr-get-lhs (lambda ()
                                    (tr-delete-indices n (tr-set i val rec))))
                  (equal-by-tr-get-rhs (lambda ()
@@ -933,14 +908,14 @@
 (defthm array-rec-pair-p-of-update-nth
   (implies (and (array-rec-pair-p arr rec len)
                 (force (natp n))
-                (force (posp len))
+                (force (natp len))
                 (force (< n len))
                 (force (elem-p val)))
            (array-rec-pair-p (update-nth n val arr) rec len)))
 
 (defthm array-rec-pair-p-of-tr-delete-indices
   (implies (array-rec-pair-p arr rec len)
-           (array-rec-pair-p arr (tr-delete-indices (- len 1) rec) len)))
+           (array-rec-pair-p arr (tr-delete-indices len rec) len)))
 
 
 
@@ -992,12 +967,12 @@
    (local (defthmd main-lemma
             (implies (and (not (equal arr1 arr2))
                           (equal (len arr1) (len arr2))
-                          (posp (len arr1))
+                          (natp (len arr1))
                           (elem-list-p arr1)
                           (elem-list-p arr2))
                      (let ((key (nth-badguy arr1 arr2)))
-                       (not (equal (tr-get key (array-to-tr (+ -1 (len arr1)) arr1 rec1))
-                                   (tr-get key (array-to-tr (+ -1 (len arr1)) arr2 rec2))))))
+                       (not (equal (tr-get key (array-to-tr (len arr1) arr1 rec1))
+                                   (tr-get key (array-to-tr (len arr1) arr2 rec2))))))
             :hints(("Goal"
                     :do-not '(generalize fertilize)
                     :do-not-induct t))))
@@ -1005,11 +980,11 @@
    (defthm equal-of-array-to-tr-part1
      (implies (and (not (equal arr1 arr2))
                    (equal (len arr1) (len arr2))
-                   (posp (len arr1))
+                   (natp (len arr1))
                    (elem-list-p arr1)
                    (elem-list-p arr2))
-              (not (equal (array-to-tr (+ -1 (len arr1)) arr1 rec1)
-                          (array-to-tr (+ -1 (len arr1)) arr2 rec2))))
+              (not (equal (array-to-tr (len arr1) arr1 rec1)
+                          (array-to-tr (len arr1) arr2 rec2))))
      :hints(("Goal"
              :do-not '(generalize fertilize)
              :do-not-induct t
@@ -1050,7 +1025,7 @@
                       (equal rec2 (tr-delete-indices n rec2))
                       (natp n)
                       (natp key))
-                 (< n key)))
+                 (<= n key)))
       :hints(("Goal"
               :do-not '(generalize fertilize)
               :do-not-induct t
@@ -1114,22 +1089,23 @@
          ;; to unify with this and apply this rule.  The rule below is better.
          (implies (and (array-rec-pair-p arr1 rec1 len)
                        (array-rec-pair-p arr2 rec2 len)
-                       (posp len))
-                  (equal (equal (array-to-tr (+ -1 len) arr1 rec1)
-                                (array-to-tr (+ -1 len) arr2 rec2))
+                       (natp len))
+                  (equal (equal (array-to-tr len arr1 rec1)
+                                (array-to-tr len arr2 rec2))
                          (and (equal arr1 arr2)
                               (equal rec1 rec2))))
          :hints(("Goal"
                  :do-not '(generalize fertilize)
                  :do-not-induct t
                  :in-theory (e/d (array-rec-pair-p)
-                                 (tr-delete-indices))))))
+                                 (tr-delete-indices))
+                 :cases ((equal rec1 rec2))))))
 
 (defthm equal-of-array-to-tr
   (implies (and (array-rec-pair-p arr1 rec1 len1)
                 (array-rec-pair-p arr2 rec2 len2)
-                (equal len1 (+ 1 len))
-                (equal len2 (+ 1 len))
+                (equal len1 len)
+                (equal len2 len)
                 (natp len))
            (equal (equal (array-to-tr len arr1 rec1)
                          (array-to-tr len arr2 rec2))
@@ -1139,7 +1115,7 @@
           :in-theory (disable tr-delete-indices
                               array-to-tr
                               array-rec-pair-p)
-          :use ((:instance equal-of-array-to-tr-orig (len (+ 1 len)))))))
+          :use ((:instance equal-of-array-to-tr-orig)))))
 
 
 
