@@ -35,7 +35,7 @@ vl::primitives)."
   :long "<p>The patterns used for the :i and :o ports here might look strange.
 For instance, why do we use @('((|a|) (|b|))') as the input pattern for an AND
 gate instead of @('(|a| |b|)')?  This allows our primitives to be directly
-compatible with VL's primitives, as far as @(see vl-portdecls-to-i/o) is
+compatible with VL's primitives, as far as @(see vl::vl-portdecls-to-i/o) is
 concerned.</p>
 
 <p><b>BOZO</b> Things to consider:</p>
@@ -193,6 +193,21 @@ than a @(see *esim-xnor*).</p>"
   :o ((|out|))
   :x (:out ((|out| . (iff |a| |b|)))))
 
+(def-esim-primitive *esim-safe-mux*
+  :short "Primitive E module for a (more conservative) mux."
+  :long "<p>We use this to implement @(see vl::*vl-1-bit-approx-mux*).</p>"
+  :i ((|sel|) (|a|) (|b|))
+  :o ((|out|))
+  :x (:out ((|out| . (ite* |sel| |a| |b|)))))
+
+(def-esim-primitive *esim-unsafe-mux*
+  :short "Primitive E module for a (less conservative) mux."
+  :long "<p>We use this to implement @(see vl::*vl-1-bit-mux*).</p>"
+  :i ((|sel|) (|a|) (|b|))
+  :o ((|out|))
+  :x (:out ((|out| . (ite |sel| |a| |b|)))))
+
+
 ;; Note: VL now generates RES modules directly, see the vl/toe directory.
 ;; BOZO perhaps rework how that's done, and perhaps WOR, WAND, etc.
 
@@ -237,6 +252,60 @@ inputs, used to support experimental esim decomposition.</p>"
   :o ((|o|))
   :x (:out ((|o| . (zif |sel| |a| |b|)))))
 
+(def-esim-primitive *esim-bufif0*
+  :short "Primitive E module for a kind of bufif0 gate."
+  :long "<p>We use this to implement @(see vl::*vl-1-bit-bufif0*).</p>"
+  :i ((|data|) (|ctrl|))
+  :o ((|out|))
+  :x (:out ((|out| . (tristate (not |ctrl|) |data|)))))
+
+(def-esim-primitive *esim-bufif1*
+  :short "Primitive E module for a kind of bufif1 gate."
+  :long "<p>We use this to implement @(see vl::*vl-1-bit-bufif1*).</p>"
+  :i ((|data|) (|ctrl|))
+  :o ((|out|))
+  :x (:out ((|out| . (tristate |ctrl| |data|)))))
+
+(def-esim-primitive *esim-notif0*
+  :short "Primitive E module for a kind of notif0 gate."
+  :long "<p>We use this to implement @(see vl::*vl-1-bit-notif0*).</p>"
+  :i ((|data|) (|ctrl|))
+  :o ((|out|))
+  :x (:out ((|out| . (tristate (not |ctrl|) (not |data|))))))
+
+(def-esim-primitive *esim-notif1*
+  :short "Primitive E module for a kind of notif1 gate."
+  :long "<p>We use this to implement @(see vl::*vl-1-bit-notif1*).</p>"
+  :i ((|data|) (|ctrl|))
+  :o ((|out|))
+  :x (:out ((|out| . (tristate |ctrl| (not |data|))))))
+
+(def-esim-primitive *esim-nmos*
+  :short "Primitive E module for a kind of nmos gate."
+  :long "<p>We use this to implement @(see vl::*vl-1-bit-nmos*).</p>"
+  :i ((|data|) (|ctrl|))
+  :o ((|out|))
+  :x (:out ((|out| . (zif |ctrl| |data| (z))))))
+
+(def-esim-primitive *esim-pmos*
+  :short "Primitive E module for a kind of pmos gate."
+  :long "<p>We use this to implement @(see vl::*vl-1-bit-pmos*).</p>"
+  :i ((|data|) (|ctrl|))
+  :o ((|out|))
+  :x (:out ((|out| . (zif |ctrl| (z) |data|)))))
+
+(def-esim-primitive *esim-cmos*
+  :short "Primitive E module for a kind of cmos gate."
+  :long "<p>We use this to implement @(see vl::*vl-1-bit-cmos*).</p>"
+  :i ((|data|) (|nctrl|) (|pctrl|))
+  :o ((|out|))
+  :x (:out ((|out| . (res (zif |nctrl| |data| (z))
+                          (zif |pctrl| (z) |data|))))))
+
+
+
+
+
 (defsection *esim-primitives*
   :parents (esim-primitives)
   :short "A list of all esim primitives."
@@ -263,5 +332,13 @@ inputs, used to support experimental esim decomposition.</p>"
           *esim-flop*
           *esim-latch*
           *esim-fsmreg*
-          *esim-zif*)))
+          *esim-zif*
+
+          *esim-bufif0*
+          *esim-bufif1*
+          *esim-notif0*
+          *esim-notif1*
+          *esim-nmos*
+          *esim-pmos*
+          *esim-cmos*)))
 
