@@ -29,8 +29,8 @@
 ;; (include-book "../follow-backwards")
 (include-book "centaur/misc/vecs-ints" :dir :system)
 (include-book "centaur/misc/tuplep" :dir :system)
-(include-book "cutil/defmvtypes" :dir :system)
-(include-book "cutil/defprojection" :dir :system)
+(include-book "std/util/defmvtypes" :dir :system)
+(include-book "std/util/defprojection" :dir :system)
 (include-book "str/natstr" :dir :system)
 (include-book "std/lists/final-cdr" :dir :system)
 (include-book "centaur/vl/util/defs" :dir :system)
@@ -142,7 +142,7 @@ turned into a list like @('(|opcode[0]| ... |opcode[n]|)').</li>
               (consp name))
          "The name of this input, and should be a list of E input bits in
           lsb-first order.  (That is, Verilog-style names should have already
-          been expanded away using @(see stv-expand-names) or similar.)")
+          been expanded away using @(see stv-expand).)")
 
    (width (equal width (len name))
           "Just the pre-computed width of this input.")
@@ -388,7 +388,7 @@ that we generate from their names.  That is, a simulation variable like
               (consp name))
          "The name of this output.  It should be a list of E input bits in
           lsb-first order.  That is, Verilog-style names should have already
-          been expanded away using @(see stv-expand-names) or similar.")
+          been expanded away using @(see stv-expand).")
 
    (width (equal width (len name))
           "Just the pre-computed width of this output.  It must be exactly
@@ -786,7 +786,7 @@ given a list of instance names and the name of the state bit."
                        "!"
                        (stv-forge-state-bit (cdr instnames) st-name))))
 
-(cutil::defprojection stv-forge-state-bits (instnames x)
+(std::defprojection stv-forge-state-bits (instnames x)
   (stv-forge-state-bit instnames x)
   :guard (atom-listp x)
   :result-type atom-listp)
@@ -818,13 +818,15 @@ given a list of instance names and the name of the state bit."
   (stv-forge-state-bit (stv-path-to-override-decision-instnames path)
                        "S"))
 
-(cutil::defprojection stv-paths-to-override-value-stbits (x)
+(std::defprojection stv-paths-to-override-value-stbits (x)
   (stv-path-to-override-value-stbit x)
-  :guard t)
+  :guard t
+  :parents nil)
 
-(cutil::defprojection stv-paths-to-override-decision-stbits (x)
+(std::defprojection stv-paths-to-override-decision-stbits (x)
   (stv-path-to-override-decision-stbit x)
-  :guard t)
+  :guard t
+  :parents nil)
 
 
 
@@ -936,7 +938,7 @@ evaluation, debugging, etc."
   ((stv stvdata-p
         "An @(see stvdata-p) that has already had its lines widened and any
          Verilog-style names expanded; see @(see stv-widen) and @(see
-         stv-expand-names).")
+         stv-expand).")
 
    (mod good-esim-modulep
         "The @(see esim) module this STV is about."))
