@@ -362,7 +362,7 @@ evaluate STVs.  A picture of the flow is:</p>
 <li>The <b>User-Level STV</b>, which is a symbolic test vector written in the
 @(see symbolic-test-vector-format),</li>
 
-<li>The <b>ESIM Module</b> to simulate, which is generally produce by VL.</li>
+<li>The <b>ESIM Module</b> to simulate, which is generally produced by VL.</li>
 
 <li>The <b>Simulation Input Alist</b>, which is an alist that should bind the
 simulation variables of the symbolic test vector to concrete, natural-numbered
@@ -786,10 +786,10 @@ some kind of ALU then we probably want to @(':mix') the @('abus') and
 suggested by the initial and input lines, and doesn't smartly mix together
 signals.</dd>
 
-<dd>If your module is small or you're using @(see gl-aig-mode), then this may
-be fine and very convenient.  For more complex modules, you'll probably want to
-write your own binding macros.  See @(see stv-easy-bindings) for a high-level
-way to describe most kind of bindings.</dd>
+<dd>If your module is small or you're using @(see gl::gl-satlink-mode), then
+this may be fine and very convenient.  For more complex modules, you'll
+probably want to write your own binding macros.  See @(see stv-easy-bindings)
+for a high-level way to describe most kind of bindings.</dd>
 
 <dt>@('(my-run-mod)')</dt>
 
