@@ -25,7 +25,7 @@
 (include-book "eval-g-base")
 (local (include-book "symbolic-arithmetic"))
 (local (include-book "eval-g-base-help"))
-(local (include-book "hyp-fix-logic"))
+(local (include-book "hyp-fix"))
 
 (local (in-theory (disable acl2::revappend-removal)))
 
@@ -124,51 +124,58 @@
 
 
 ;; Collect concrete characters onto "pre".
-(defun coerce-string (x pre hyp)
-  (declare (xargs :guard (and (true-listp pre))
-                  :verify-guards nil))
-  (if (or (general-concretep x) (atom x))
-      (mk-g-concrete
-       (ec-call (coerce (revappend pre (general-concrete-obj x)) 'string)))
-    (pattern-match x
-      ((g-ite test then else)
-       (g-if test
-             (coerce-string then pre hyp)
-             (coerce-string else pre hyp)))
-      ((g-apply & &) (g-apply 'coerce (gl-list (revappend-concrete pre x) 'string)))
-      ((g-var &) (g-apply 'coerce (gl-list (revappend-concrete pre x) 'string)))
-      ((g-number &)
-       (mk-g-concrete
-        (ec-call (coerce (revappend pre nil) 'string))))
-      ((g-boolean &)
-       (mk-g-concrete
-        (ec-call (coerce (revappend pre nil) 'string))))
-      ((g-concrete obj)
-       ;; Actually taken care of by the first IF.
-       (mk-g-concrete
-        (ec-call (coerce (revappend pre obj) 'string))))
-      (& (if (or (general-concretep (car x))
-                 (atom (car x)))
-             (coerce-string (cdr x) (cons (general-concrete-obj (car x)) pre)
-                            hyp)
-           (pattern-match (car x)
-             ((g-ite test then else)
-              (g-if test
-                    (coerce-string (gl-cons then (cdr x)) pre hyp)
-                    (coerce-string (gl-cons else (cdr x)) pre hyp)))
-             ((g-apply & &) (g-apply 'coerce (gl-list (revappend-concrete pre x)
-                                                   'string)))
-             ((g-var &) (g-apply 'coerce (gl-list (revappend-concrete pre x)
-                                               'string)))
-             (&
-              (coerce-string (cdr x) (cons (code-char 0) pre) hyp))))))))
+(define coerce-string (x pre hyp)
+  :guard (true-listp pre)
+  :verify-guards nil
+  (b* ((hyp (lbfr-hyp-fix hyp)))
+    (replace-g-ifs
+     (if (or (general-concretep x) (atom x))
+         (gret (mk-g-concrete
+                (ec-call (coerce (revappend pre (general-concrete-obj x)) 'string))))
+       (pattern-match x
+         ((g-ite test then else)
+          (g-if (gret test)
+                (coerce-string then pre hyp)
+                (coerce-string else pre hyp)))
+         ((g-apply & &) (gret (g-apply 'coerce (gl-list (revappend-concrete pre x) 'string))))
+         ((g-var &) (gret (g-apply 'coerce (gl-list (revappend-concrete pre x) 'string))))
+         ((g-number &)
+          (gret (mk-g-concrete
+                 (ec-call (coerce (revappend pre nil) 'string)))))
+         ((g-boolean &)
+          (gret (mk-g-concrete
+                 (ec-call (coerce (revappend pre nil) 'string)))))
+         ((g-concrete obj)
+          ;; Actually taken care of by the first IF.
+          (gret (mk-g-concrete
+                 (ec-call (coerce (revappend pre obj) 'string)))))
+         (& (if (or (general-concretep (car x))
+                    (atom (car x)))
+                (coerce-string (cdr x) (cons (general-concrete-obj (car x)) pre)
+                               hyp)
+              (pattern-match (car x)
+                ((g-ite test then else)
+                 (g-if (gret test)
+                       (coerce-string (gl-cons then (cdr x)) pre hyp)
+                       (coerce-string (gl-cons else (cdr x)) pre hyp)))
+                ((g-apply & &) (gret (g-apply 'coerce (gl-list (revappend-concrete pre x)
+                                                               'string))))
+                ((g-var &) (gret (g-apply 'coerce (gl-list (revappend-concrete pre x)
+                                                           'string))))
+                (&
+                 (coerce-string (cdr x) (cons (code-char 0) pre) hyp)))))))))
+  ///
+  (def-hyp-congruence coerce-string)
+  (verify-guards coerce-string
+    :hints(("Goal" :in-theory (e/d (g-if-fn g-or-fn)
+                                   (coerce-string)))))
 
-(defthm deps-of-coerce-string
-  (implies (not (gobj-depends-on k p x))
-           (not (gobj-depends-on k p (coerce-string x pre hyp))))
-  :hints (("goal" :induct (coerce-string x pre hyp)
-           :expand ((coerce-string x pre hyp))
-           :in-theory (disable (:d coerce-string)))))
+  (defthm deps-of-coerce-string
+    (implies (not (gobj-depends-on k p x))
+             (not (gobj-depends-on k p (mv-nth 0 (coerce-string x pre hyp)))))
+    :hints (("goal" :induct (coerce-string x pre hyp)
+             :expand ((coerce-string x pre hyp))
+             :in-theory (disable (:d coerce-string))))))
 
 
 (local
@@ -200,12 +207,6 @@
    ;;                 (gobjectp (cdr x)))))
    ))
 
-(verify-guards
- coerce-string
- :hints(("Goal" :in-theory (e/d (atom-impl-general-concretep-1)
-                                (coerce-string
-                                 )))))
-
 
 (local
  (encapsulate
@@ -329,13 +330,14 @@
 
 
    (defthm coerce-string-correct
-     (implies (bfr-eval hyp (car env))
-              (equal (eval-g-base (coerce-string x pre hyp) env)
+     (implies (bfr-hyp-eval hyp (car env))
+              (equal (eval-g-base (mv-nth 0 (coerce-string x pre hyp)) env)
                      (coerce (revappend pre (eval-g-base x env)) 'string)))
      :hints(("Goal" :in-theory (e/d* ( general-concrete-obj
                                        concrete-gobjectp-def
                                        eval-g-base-list
-                                       general-concretep-def)
+                                       general-concretep-def
+                                       (:i coerce-string))
                                      ((:definition coerce-string)
                                       member eval-g-base
                                       equal-of-booleans-rewrite
@@ -365,27 +367,31 @@
                          ))
                    :expand ((:with eval-g-base (eval-g-base (car x) env)))))))))
 
-(defun coerce-list (x hyp)
-  (declare (xargs :guard t
-                  :verify-guards nil))
-  (if (or (general-concretep x) (atom x))
-      (mk-g-concrete
-       (ec-call (coerce (general-concrete-obj x) 'list)))
-    (pattern-match x
-      ((g-ite test then else)
-       (g-if test
-             (coerce-list then hyp)
-             (coerce-list else hyp)))
-      ((g-apply & &) (g-apply 'coerce (gl-list x 'list)))
-      ((g-var &) (g-apply 'coerce (gl-list x 'list)))
-      (& nil))))
+(define coerce-list (x hyp)
+  :verify-guards nil
+  (b* ((hyp (lbfr-hyp-fix hyp)))
+    (if (or (general-concretep x) (atom x))
+        (gret (mk-g-concrete
+               (ec-call (coerce (general-concrete-obj x) 'list))))
+      (pattern-match x
+        ((g-ite test then else)
+         (g-if-mbe (gret test)
+                   (coerce-list then hyp)
+                   (coerce-list else hyp)))
+        ((g-apply & &) (gret (g-apply 'coerce (gl-list x 'list))))
+        ((g-var &) (gret (g-apply 'coerce (gl-list x 'list))))
+        (& (gret nil)))))
+  ///
+  (def-hyp-congruence coerce-list)
+  (verify-guards coerce-list
+    :hints(("Goal" :in-theory (enable g-if-fn))))
 
-(defthm deps-of-coerce-list
-  (implies (not (gobj-depends-on k p x))
-           (not (gobj-depends-on k p (coerce-list x hyp))))
-  :hints (("goal" :induct  (coerce-list x hyp)
-           :expand ((coerce-list x hyp))
-           :in-theory (disable (:d coerce-list)))))
+  (defthm deps-of-coerce-list
+    (implies (not (gobj-depends-on k p x))
+             (not (gobj-depends-on k p (mv-nth 0 (coerce-list x hyp)))))
+    :hints (("goal" :induct  (coerce-list x hyp)
+             :expand ((coerce-list x hyp))
+             :in-theory (disable (:d coerce-list))))))
 
 
 ;; (local
@@ -397,10 +403,7 @@
 ;;             :induct (coerce-list x hyp)
 ;;             :expand ((coerce-list x hyp))))))
 
-(verify-guards
- coerce-list
- :hints(("Goal" :in-theory (e/d (atom-impl-general-concretep-1)
-                                (coerce-list)))))
+
 
 
 (encapsulate nil
@@ -421,12 +424,13 @@
 
 
   (defthm coerce-list-correct
-    (implies (and (bfr-eval hyp (car env)))
-             (equal (eval-g-base (coerce-list x hyp) env)
+    (implies (and (bfr-hyp-eval hyp (car env)))
+             (equal (eval-g-base (mv-nth 0 (coerce-list x hyp)) env)
                     (coerce (eval-g-base x env) 'list)))
     :hints(("Goal" :in-theory (e/d* (; (:ruleset g-correct-lemmas)
                                      eval-g-base-general-concrete-obj
-                                     eval-g-base eval-g-base-list)
+                                     eval-g-base eval-g-base-list
+                                     (:i coerce-list))
                                     ((:definition coerce-list)
                                      eval-g-base-alt-def))
             :induct (coerce-list x hyp)
@@ -444,14 +448,14 @@
      (pattern-match y
        ((g-ite ytest ythen yelse)
         (if (zp clk)
-            (g-apply 'coerce (gl-list x y))
-          (g-if ytest
+            (gret (g-apply 'coerce (gl-list x y)))
+          (g-if (gret ytest)
                 (,gfn x ythen hyp clk config bvar-db state)
                 (,gfn x yelse hyp clk config bvar-db state))))
        ((g-apply & &)
-        (g-apply 'coerce (gl-list x y)))
+        (gret (g-apply 'coerce (gl-list x y))))
        ((g-var &)
-        (g-apply 'coerce (gl-list x y)))
+        (gret (g-apply 'coerce (gl-list x y))))
        (& (coerce-string x nil hyp)))))
 
 
