@@ -25,7 +25,7 @@
 (include-book "eval-g-base")
 (local (include-book "symbolic-arithmetic"))
 (local (include-book "eval-g-base-help"))
-(local (include-book "hyp-fix-logic"))
+(local (include-book "hyp-fix"))
 (set-inhibit-warnings "theory")
 
 ;; (defthm true-listp-of-bfr-integer-length-s1
@@ -40,33 +40,33 @@
 
 
 (def-g-fn integer-length
-  `(let ((x i))
+  `(b* ((x i))
      (if (atom x)
-         (ec-call (integer-length x))
+         (gret (ec-call (integer-length x)))
        (pattern-match x
          ((g-ite test then else)
           (if (zp clk)
-              (g-apply 'integer-length (gl-list x))
-            (g-if test
+              (gret (g-apply 'integer-length (gl-list x)))
+            (g-if (gret test)
                   (,gfn then . ,params)
                   (,gfn else . ,params))))
          ((g-apply & &)
-          (g-apply 'integer-length (gl-list x)))
+          (gret (g-apply 'integer-length (gl-list x))))
          ((g-var &)
-          (g-apply 'integer-length (gl-list x)))
-         ((g-boolean &) 0)
-         ((g-concrete obj) (ec-call (integer-length obj)))
+          (gret (g-apply 'integer-length (gl-list x))))
+         ((g-boolean &) (gret 0))
+         ((g-concrete obj) (gret (ec-call (integer-length obj))))
          ((g-number num)
           (mv-let (arn ard ain aid)
             (break-g-number num)
-            (g-if (mk-g-boolean (hyp-fix (bfr-or (bfr-=-uu aid nil)
-                                               (bfr-=-ss ain nil)) hyp))
-                  (g-if (equal ard '(t))
+            (g-if (gret (mk-g-boolean (hyp-fix (bfr-or (bfr-=-uu aid nil)
+                                                       (bfr-=-ss ain nil)) hyp)))
+                  (g-if (gret (equal ard '(t)))
                         (let ((res (rlist-fix (bfr-integer-length-s arn))))
-                          (mk-g-number res 1 0 1))
-                        (g-apply 'integer-length (gl-list x)))
-                  0)))
-         (& 0)))))
+                          (gret (mk-g-number res 1 0 1)))
+                        (gret (g-apply 'integer-length (gl-list x))))
+                  (gret 0))))
+         (& (gret 0))))))
 
 ;; (local (defthm gobjectp-integer-length
 ;;          (gobjectp (integer-length a))
@@ -116,6 +116,7 @@
                   (equal (eval-g-base x env) x))
          :hints(("Goal" :in-theory (enable eval-g-base)))))
 
+
 (def-g-correct-thm integer-length eval-g-base
   :hints `(("Goal" :in-theory (e/d (components-to-number-alt-def)
                                    ((:definition ,gfn)
