@@ -25,7 +25,7 @@
 (include-book "eval-g-base")
 (local (include-book "symbolic-arithmetic"))
 (local (include-book "eval-g-base-help"))
-(local (include-book "hyp-fix-logic"))
+(local (include-book "hyp-fix"))
 
 (defun g-<-of-numbers (a b)
   (declare (xargs :guard (and (general-numberp a)
@@ -122,7 +122,7 @@
 (def-g-binary-op <
   (b* ((x-num (if (general-numberp x) x 0))
        (y-num (if (general-numberp y) y 0)))
-    (g-<-of-numbers x-num y-num)))
+    (gret (g-<-of-numbers x-num y-num))))
 
 ;; (def-gobjectp-thm <
 ;; :hints `(("Goal" :in-theory (e/d* (booleanp-gobjectp
