@@ -25,7 +25,7 @@
 (include-book "eval-g-base")
 (local (include-book "symbolic-arithmetic"))
 (local (include-book "eval-g-base-help"))
-(local (include-book "hyp-fix-logic"))
+(local (include-book "hyp-fix"))
 (set-inhibit-warnings "theory")
 
 (defun g-logbitp-of-numbers (a b)
@@ -103,7 +103,7 @@
 (def-g-binary-op logbitp
   (b* ((i-num (if (general-numberp i) i 0))
        (j-num (if (general-numberp j) j 0)))
-    (g-logbitp-of-numbers i-num j-num)))
+    (gret (g-logbitp-of-numbers i-num j-num))))
 
 ;; (def-gobjectp-thm logbitp
 ;;   :hints `(("Goal" :in-theory
