@@ -25,7 +25,7 @@
 (include-book "eval-g-base")
 (local (include-book "symbolic-arithmetic"))
 (local (include-book "eval-g-base-help"))
-(local (include-book "hyp-fix-logic"))
+(local (include-book "hyp-fix"))
 (local (include-book "centaur/misc/arith-equivs" :dir :system))
 
 (defun g-binary-logior-of-numbers (x y)
@@ -96,7 +96,7 @@
 (def-g-binary-op binary-logior
   (b* ((i-num (if (general-numberp i) i 0))
        (j-num (if (general-numberp j) j 0)))
-    (g-binary-logior-of-numbers i-num j-num)))
+    (gret (g-binary-logior-of-numbers i-num j-num))))
 
 
 
