@@ -25,7 +25,7 @@
 (include-book "eval-g-base")
 (local (include-book "symbolic-arithmetic"))
 (local (include-book "eval-g-base-help"))
-(local (include-book "hyp-fix-logic"))
+(local (include-book "hyp-fix"))
 
 (defun g-rem-of-numbers (x y)
   (declare (xargs :guard (and (general-numberp x)
@@ -91,7 +91,7 @@
 (def-g-binary-op rem
   (b* ((x-num (if (general-numberp x) x 0))
        (y-num (if (general-numberp y) y 0)))
-    (g-rem-of-numbers x-num y-num)))
+    (gret (g-rem-of-numbers x-num y-num))))
 
 (verify-g-guards
  rem
