@@ -25,7 +25,7 @@
 (include-book "eval-g-base")
 (local (include-book "symbolic-arithmetic"))
 (local (include-book "eval-g-base-help"))
-(local (include-book "hyp-fix-logic"))
+(local (include-book "hyp-fix"))
 (local (include-book "arithmetic/top-with-meta" :dir :system))
 
 (defun g-truncate-of-numbers (x y)
@@ -79,7 +79,7 @@
 (def-g-binary-op truncate
   (b* ((i-num (if (general-numberp i) i 0))
        (j-num (if (general-numberp j) j 0)))
-    (g-truncate-of-numbers i-num j-num)))
+    (gret (g-truncate-of-numbers i-num j-num))))
 
 
 (verify-g-guards
