@@ -25,35 +25,36 @@
 (include-book "eval-g-base")
 (local (include-book "symbolic-arithmetic"))
 (local (include-book "eval-g-base-help"))
-(local (include-book "hyp-fix-logic"))
+(local (include-book "hyp-fix"))
 (local (include-book "arithmetic/top-with-meta" :dir :system))
 (set-inhibit-warnings "theory")
 
 (def-g-fn unary--
   `(if (atom x)
-       (- (fix x))
+       (gret (- (fix x)))
      (pattern-match x
        ((g-ite test then else)
         (if (zp clk)
-            (g-apply 'unary-- (gl-list x))
-          (g-if test
+            (gret (g-apply 'unary-- (gl-list x)))
+          (g-if (gret test)
                 (,gfn then . ,params)
                 (,gfn else . ,params))))
        ((g-apply & &)
-        (g-apply 'unary-- (gl-list x)))
+        (gret (g-apply 'unary-- (gl-list x))))
        ((g-concrete obj)
-        (- (fix obj)))
+        (gret (- (fix obj))))
        ((g-var &)
-        (g-apply 'unary-- (gl-list x)))
-       ((g-boolean &) 0)
+        (gret (g-apply 'unary-- (gl-list x))))
+       ((g-boolean &) (gret 0))
        ((g-number num)
         (mv-let (rn rd in id)
           (break-g-number num)
-          (mk-g-number (bfr-unary-minus-s rn)
-                       (rlist-fix rd)
-                       (bfr-unary-minus-s in)
-                       (rlist-fix id))))
-       (& 0))))
+          (gret
+           (mk-g-number (bfr-unary-minus-s rn)
+                        (rlist-fix rd)
+                        (bfr-unary-minus-s in)
+                        (rlist-fix id)))))
+       (& (gret 0)))))
 
 ;; (def-gobjectp-thm unary--
 ;;   :hints `(("Goal" :in-theory (e/d () ((:definition ,gfn)))
