@@ -1,6 +1,24 @@
+; GL - A Symbolic Simulation Framework for ACL2
+; Copyright (C) 2008-2013 Centaur Technology
+;
+; Contact:
+;   Centaur Technology Formal Verification Group
+;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
+;   http://www.centtech.com/
+;
+; This program is free software; you can redistribute it and/or modify it under
+; the terms of the GNU General Public License as published by the Free Software
+; Foundation; either version 2 of the License, or (at your option) any later
+; version.  This program is distributed in the hope that it will be useful but
+; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+; more details.  You should have received a copy of the GNU General Public
+; License along with this program; if not, write to the Free Software
+; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+;
+; Original author: Sol Swords <sswords@centtech.com>
 
 (in-package "GL")
-
 (include-book "defagg")
 (include-book "tools/pattern-match" :dir :system)
 (include-book "misc/untranslate-patterns" :dir :system)
@@ -9,7 +27,7 @@
 (defagg g-boolean (bool))
 (defagg g-number (num))
 (defagg g-ite (test then else))
-(defagg g-apply (fn args))
+(defagg g-apply (fn args) :notinline t)
 (defagg g-var (name))
 
 (defconst *g-keywords*
@@ -98,3 +116,10 @@
   :rule-classes ((:rewrite :backchain-limit-lst 0)
                  :type-prescription))
 
+(defun gl-cons (x y)
+  (declare (xargs :guard t))
+  (cons (if (g-keyword-symbolp x)
+            (g-concrete x)
+          x)
+        y))
+
