@@ -45,7 +45,7 @@ Developing faster SAT solvers is an active area of research with frequent <a
 href='http://www.satcompetition.org/'>competitions</a>.</p>
 
 <p>SAT solvers are useful because many problems can be recast as SAT problems.
-For instance, the @(see gl::gl) framework can translate many ACL2 proof goals,
+For instance, the @(see gl) framework can translate many ACL2 proof goals,
 e.g., bounded arithmetic problems, into SAT problems.  This allows for a large
 class of theorems to be solved quickly and automatically by the SAT solver.
 This is especially useful in @(see acl2::hardware-verification).</p>
@@ -53,8 +53,8 @@ This is especially useful in @(see acl2::hardware-verification).</p>
 <p><b>Satlink</b> is an interfacing library that allows ACL2 to make use of
 off-the-shelf SAT solvers like <a
 href='http://fmv.jku.at/lingeling/'>lingeling</a>, <a
-href='https://www.lri.fr/~simon/?page=glucose'>glucose</a>, and so on.  It
-provides:</p>
+href='https://www.lri.fr/~simon/?page=glucose'>glucose</a>, and so on; see
+@(see sat-solver-options).  It provides:</p>
 
 <ul>
 
@@ -75,12 +75,19 @@ SAT solver claims the formula is unsatisfiable, it's correct.</li>
 is satisfiable, since we can just check the alleged satisfying assignment it
 produces.</p>
 
+<p>The soundness threat may be reduced by checking the output of the SAT
+solver.  There has been recent progress in the SAT community toward
+standardizing formats for UNSAT proofs, reducing the overhead of producing
+these proofs, and developing fast tools to check these proofs.  Satlink
+includes, e.g., a @('glucose-cert') script that implements this idea; see @(see
+sat-solver-options).</p>
+
 
 <h3>Loading the Library</h3>
 
 <p>Satlink is a low-level library.  It would be a bit weird to want to use it
 directly.  Instead, it is typically used indirectly through other tools, such
-as the @(see gl::gl) framework, or @(see acl2::aig-sat), or the @(see
+as the @(see gl) framework, or @(see acl2::aig-sat), or the @(see
 aignet::aignet-cnf) translation.  These other approaches are likely to be more
 convenient than using Satlink directly.</p>
 
@@ -95,8 +102,6 @@ is:</p>
 cnf) formula in some way, and then call @(see sat).</p>
 
 
-
-
 <h3>Copyright Information</h3>
 
 <p>Satlink &mdash; Link from ACL2 to SAT Solvers<br/>
@@ -123,6 +128,269 @@ details.</p>
 this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
 Street, Suite 500, Boston, MA 02110-1335, USA.</p>")
 
+(defxdoc sat-solver-options
+  :parents (satlink)
+  :short "How to get a SAT solver that you can use with @(see satlink)."
+
+  :long "<p>To use Satlink you will need at least one SAT solver.  Below are
+some solvers that are known to work well with Satlink.  You may wish to install
+several of these.  This lets you switch between solvers to figure out which
+solver is best for your particular problem, and perhaps to gain some additional
+confidence that a particular problem really is unsatisfiable (by checking it
+with many solvers.)</p>
+
+
+<h3><a href='http://www.labri.fr/perso/lsimon/glucose/'>Glucose</a> &mdash;
+open source, recommended</h3>
+
+<p>Based on our experiences using @(see gl) for proofs about hardware modules
+at Centaur, we usually try Glucose first.  Version 3.0 should work with Satlink
+without any modifications.  (We have also successfully used earlier versions
+with Satlink, but occasionally needed to patch them in minor ways, e.g., to
+print counterexamples.)</p>
+
+<p>Quick instructions:</p>
+
+<ul>
+<li>Download Glucose 3.0 and extract @('glucose-3.0.tgz') somewhere</li>
+<li>@('cd glucose-3.0/core; make')</li>
+<li>@('cd glucose-3.0/simp; make')</li>
+<li>Verify that @('glucose-3.0/simp/glucose --help') prints a help message</li>
+</ul>
+
+<p>Now create a shell script, somewhere in your @('$PATH'), named @('glucose'):</p>
+
+@({
+    #!/bin/sh
+    /path/to/glucose-3.0/simp/glucose -model \"$@\"
+})
+
+<p>As a quick test to make sure things are working, you can try to certify this
+book:</p>
+
+@({
+    cd [books]/centaur/satlink/solvers
+    cert.pl test-glucose
+})
+
+<p>You should now be able to use Satlink with configurations such as:</p>
+
+@({
+    (satlink::make-config :cmdline \"glucose\")
+})
+
+<p>Glucose can (optionally) produce UNSAT proofs that can be checked by a
+simpler program.  Satlink includes a script that makes it easy to run the
+solver and then check any UNSAT answers.  See @(see unsat-checking) for more
+information.</p>
+
+
+
+<h3><a href='http://www.cril.univ-artois.fr/~hoessen/penelope.html'>Penelope</a></h3>
+
+<p>Penelope is a (mostly) open-source, parallel sat solver.  The build process
+is simple: run @('make') and add the resulting @('penelope') executable to your
+@('PATH').  As a quick test to ensure things are working, you can try to
+certify this book:</p>
+
+@({
+    cd [books]/centaur/satlink/solvers
+    cert.pl test-penelope
+})
+
+<p>Note that you will usually need to point Penelope at a configuration file
+that tells it, e.g., how many cores to use.  The Penelope distribution has a
+@('configuration.ini') file that you can start with.  You can then use a
+Satlink configuration like:</p>
+
+@({
+    (satlink::make-config
+     :cmdline \"penelope -config=/path/to/configuration.ini\")
+})
+
+<p>We are not aware of any UNSAT proof support for this solver.</p>
+
+
+<h3><a href='http://fmv.jku.at/lingeling/'>Lingeling</a></h3>
+
+<p>An older version of Lingeling (Version ALA) is released under an open
+source (GPL3) license.  For most of our problems, we found this version to be
+somewhat slower than Glucose 2.1.  Of course, our experiences may not be
+representative.</p>
+
+<p>Newer versions of Lingeling are available under a more restrictive license.
+These modern versions fared very well in the recent SAT competitions, so if the
+license does not pose a problem for you, it may well be worth trying.</p>
+
+<p>For the open source version ALA, the build process is simple: download and
+extract the tarball, run @('./configure') and @('make'), and then add the
+resulting @('lingeling') and/or @('plingeling') executables to your PATH.  As a
+quick test to ensure things are working, you can try to certify these
+books:</p>
+
+@({
+    cd [books]/centaur/satlink/solvers
+    cert.pl test-lingeling
+    cert.pl test-plingeling
+})
+
+<p>Newer versions of Lingeling are allegedly able to produce unsat proofs, but
+we haven't figured out how to make it work.</p>
+
+
+<h3><a
+href='http://tools.computational-logic.org/content/riss3g.php'>Riss3G</a></h3>
+
+<p>The Riss3g solver is a (mostly) open-source variant of Glucose that did
+fairly well in recent competitions and seems to be sometimes faster than
+Glucose.</p>
+
+<p>The build process was reasonably straightforward.  After building, put the
+resulting @('riss3g') and @('riss3gSimp') programs into your @('PATH').  As a
+quick test to ensure things are working, you can try to certify these
+books:</p>
+
+@({
+    cd [books]/centaur/satlink/solvers
+    cert.pl test-riss3g
+    cert.pl test-riss3gSimp
+})
+
+<p>Like Glucose, @('riss3g') has an ability to produce unsat proofs that can be
+checked with an external program.  Satlink includes a script that makes it easy
+to run the solver and then check any UNSAT answers.  See @(see unsat-checking)
+for more information.</p>
+
+
+<h3>Other Solvers</h3>
+
+<p>In principle, Satlink should work with any SAT solver that implements the
+DIMACS format.  This format is used in the <a
+href='http://www.satcompetition.org/'>SAT competitions</a>, so it is
+implemented by many solvers.  Accordingly, you may wish to look at, e.g., the
+<a href='http://www.satcompetition.org/2013/results.shtml'>SAT Competition
+Results</a> to get ideas about what SAT solvers are likely to perform well.</p>
+
+<p>Getting a new solver to work with Satlink <i>should</i> be straightforward.
+Typically you will need to install the solver, add it to your @('PATH'), and
+then figure out how to run it.  Satlink expects to be able to invoke the solver
+using:</p>
+
+@({
+     <solver-command> <input-file>
+})
+
+<p>Sometimes a solver may need extra command-line arguments.  For instance,
+Glucose needs a @('-model') switch or it won't print the satisfying
+assignment (i.e., @('v') lines) in case of SAT.  You might be able to write a
+small script to supply these arguments, e.g., as in the @('glucose') script
+above.</p>
+
+<p>To test out your new solver, Satlink includes a primitive @(see
+check-config) command that you can use to try your solver on a handful of
+trivial problems.  This is a very good way to make sure that at least the basic
+i/o contract seems to be working.  It should be easy to adapt one of the
+@('test-') scripts in the @('centaur/satlink/solvers') directory to suit your
+solver.</p>
+
+<p>If the @('check-config') passes and you want a more thorough check, you
+might try to run your new solver on, e.g., @('centaur/tutorial/sat.lsp') and
+the various files in @('centaur/regression').</p>")
+
+
+(defsection unsat-checking
+  :parents (sat-solver-options)
+  :short "Options for running SAT solvers that produce UNSAT proofs."
+
+  :long "<p>For higher confidence (at some cost to runtime), some SAT solvers
+are able to produce UNSAT proofs.  Small programs such as <a
+href='http://www.cs.utexas.edu/~marijn/drup/'>drup-trim</a> can check these
+proofs, to ensure the SAT solver reasoned correctly.</p>
+
+<p>Satlink now includes Perl scripts that can make use of this capability for
+the Glucose and Riss3g solvers.  In particular, see the following scripts:</p>
+
+<ul>
+<li>@('centaur/satlink/solvers/glucose-cert')</li>
+<li>@('centaur/satlink/solvers/riss3g-cert')</li>
+</ul>
+
+<p>The general idea of these scripts is, e.g., for Glucose:</p>
+
+<ul>
+
+<li>We first call Glucose to solve the problem;</li>
+
+<li>When Glucose reports SAT, we just exit (because Satlink can check the
+satisfying assignment itself); or</li>
+
+<li>When Glucose reports UNSAT, we check the proof using the Drup-Trim unsat
+proof checker.  We only print an \"s UNSATISFIABLE\" line if Drup-Trim says
+that the proof is ok.</li>
+
+</ul>
+
+<p>All of this works well with Satlink.  You can still see the output from the
+solver and the verifier in real time, interrupt it, etc.</p>
+
+
+<h3>Setup</h3>
+
+<p>To use these tools, you will need to first:</p>
+
+<ul>
+
+<li>Install @('glucose') and/or @('riss3g') as described in @(see
+sat-solver-options), and</li>
+
+<li>Install the <a href='http://www.cs.utexas.edu/~marijn/drup/'>drup-trim</a>
+program as @('drup-trim') somewhere in your PATH.</li>
+
+</ul>
+
+<p>You can then add the @('glucose-cert') or @('riss3g-cert') scripts to your
+PATH.  As a quick test to ensure things are working, you can try to certify
+these books:</p>
+
+@({
+    cd [books]/centaur/satlink/solvers
+    cert.pl test-glucose-cert
+    cert.pl test-riss3g-cert
+})
+
+<p>To use these scripts from Satlink, you can then typically just use a Satlink
+configuration such as:</p>
+
+@({
+    (satlink::make-config :cmdline \"glucose-cert\" ...)
+})
+
+
+<h3>Skipping Proof Checking During Development</h3>
+
+<p>The environment variable SATLINK_TRUST_SOLVER can be set to 1 to suppress
+proof checking.  When this variable is set, we will NOT instruct the solver to
+produce an UNSAT proof and (of course) will not check the non-existent
+proof.</p>
+
+<p>We use this feature as follows:</p>
+
+<ul>
+
+<li>We generally set @('SATLINK_TRUST_SOLVER=1') in our startup scripts.  This
+way, when we are working with ACL2 (either interactively, or when rebuilding
+books), we just trust the solver and avoid the overhead of UNSAT checking.</li>
+
+<li>We set @('SATLINK_TRUST_SOLVER=0') for our nightly regressions.  These are
+run automatically, overnight, so performance is not very important.</li>
+
+</ul>
+
+<p>This gives us a good blend of performance and confidence.  If the solver
+somehow screws up and claims to have proven a theorem that isn't really true,
+we may not find out about it until our regression fails.  But in exchange, we
+can always use @('glucose-cert') with no performance impact on our everyday
+work.</p>")
 
 (defsection dimacs-interp
   :parents (dimacs)
@@ -199,9 +467,13 @@ Street, Suite 500, Boston, MA 02110-1335, USA.</p>")
                    ((< index (bits-length env$))
                     (set-bit index (if minus-p 0 1) env$))
                    (t
-                    (prog2$
-                     (raise "Assignment to out-of-bounds variable: ~x0" index)
-                     env$)))))
+                    ;; Historically we caused an error here.  However, we later
+                    ;; found that some SAT solvers (e.g., Riss3g) would produce
+                    ;; assignments to other variables, presumably variables that
+                    ;; they are inventing.  Well, this isn't really a problem,
+                    ;; and we're going to check the assignment anyway, so it
+                    ;; seems reasonable to tolerate this.
+                    env$))))
     (satlink-parse-variable-line x (+ n len) xl saw-zero-p env$)))
 
 (define satlink-handle-line
@@ -219,13 +491,14 @@ Street, Suite 500, Boston, MA 02110-1335, USA.</p>")
       env$)
 
   (b* ((len (length line))
-       ((when (zp len))
-        ;; We'll just ignore blank lines
+       ((when (< len 2))
+        ;; We'll ignore blank lines and lines that don't have a "x " at the
+        ;; beginning, where x is "s" or "v".
         (mv nil saw-unsat-p saw-sat-p saw-zero-p env$))
        (char (char line 0))
-       ((when (or (eql char #\c) ;; Comment line
-                  (eql char #\t) ;; Timing line
-                  ))
+       ((unless (and (or (eql char #\s)  ;; Result
+                         (eql char #\v)) ;; Variable assignment
+                     (eql (char line 1) #\Space)))
         ;; Ignore it
         (mv nil saw-unsat-p saw-sat-p saw-zero-p env$))
        ((when (eql char #\s))
@@ -235,14 +508,8 @@ Street, Suite 500, Boston, MA 02110-1335, USA.</p>")
                (mv nil t saw-sat-p saw-zero-p env$))
               (t
                (prog2$
-                (cw "SATLINK: Don't recognize this line: ~s0~%" line)
-                (mv t saw-unsat-p saw-sat-p saw-zero-p env$)))))
-       ((when (str::strprefixp "Solved by" line))
-        ;; Ignore glucose 2.2 feature
-        (mv nil saw-unsat-p saw-sat-p saw-zero-p env$))
-       ((unless (eql char #\v))
-        (cw "SATLINK: Don't recognize this line: ~s0~%" line)
-        (mv t saw-unsat-p saw-sat-p saw-zero-p env$))
+                (cw "SATLINK: Unrecognized result line: ~s0~%" line)
+                (mv nil saw-unsat-p saw-sat-p saw-zero-p env$)))))
        ;; Else it's a variable line
        ((when saw-zero-p)
         (cw "SATLINK: Variable lines after already saw zero: ~s0~%" line)
@@ -294,8 +561,11 @@ Street, Suite 500, Boston, MA 02110-1335, USA.</p>")
         (cw "SATLINK: solver says UNSAT but is giving us variables?")
         (mv :failed env$))
        ((when saw-unsat-p)
-        (mv :unsat env$)))
-    (mv :sat env$)))
+        (mv :unsat env$))
+       ((when saw-sat-p)
+        (mv :sat env$)))
+    ;; Didn't see either sat or unsat.
+    (mv :failed env$)))
 
 
 (define satlink-run-impl
@@ -334,7 +604,10 @@ satlink-run).</p>"
        (cmd (str::cat config.cmdline " " filename))
        ((mv finishedp & lines)
         (time$ (acl2::tshell-call cmd
-                                  :print config.verbose
+                                  ;; Print only if :verbose t is on, and use a
+                                  ;; custom printing function that skips variable
+                                  ;; assignment lines
+                                  :print (and config.verbose 'satlink-echo)
                                   :save t)
                :msg "; SATLINK: `~s0`: ~st sec, ~sa bytes~%"
                :args (list cmd)
@@ -492,17 +765,10 @@ invoke @(see satlink-run-impl), which actually calls the SAT solver.</p>"
     (declare (xargs :stobjs env$
                     :guard (and (config-p config)
                                 (lit-list-listp formula))))
-    (satlink-run-fn config formula env$))
+    (satlink-run-fn config formula env$)))
 
-  (progn!
-   (set-raw-mode t)
-   (defun satlink-run (config formula env$)
-     (b* ((state acl2::*the-live-state*)
-          (prev-okp            (f-get-global 'acl2::writes-okp state))
-          (state               (f-put-global 'acl2::writes-okp t state))
-          ((mv res env$ state) (satlink-run-impl config formula env$))
-          (?state              (f-put-global 'acl2::writes-okp prev-okp state)))
-       (mv res env$)))))
+; (depends-on "top-raw.lsp")
+(acl2::include-raw "top-raw.lsp")
 
 (define sat
   :parents (satlink)
