@@ -55,13 +55,14 @@
 (defun vl-gatebldr-tuple-p (x)
   (declare (xargs :guard t))
   (and (tuplep 3 x)
-       (vl-maybe-string-p (first x)) ;; name
+       (maybe-stringp (first x)) ;; name
        (vl-maybe-range-p (second x)) ;; range
        (vl-exprlist-p (third x))))   ;; args
 
 (deflist vl-gatebldr-tuple-list-p (x)
   (vl-gatebldr-tuple-p x)
-  :elementp-of-nil nil)
+  :elementp-of-nil nil
+  :parents nil)
 
 (defund vl-build-gate-instances (loc tuples type strength delay atts)
   (declare (xargs :guard (and (vl-location-p loc)
@@ -167,7 +168,7 @@
 
 (defparser vl-parse-optional-name-of-gate-instance (tokens warnings)
   :result (and (consp val)
-               (vl-maybe-string-p (car val)) ;; the name
+               (maybe-stringp (car val)) ;; the name
                (vl-maybe-range-p (cdr val))) ;; the range
   :fails gracefully
   :count weak
