@@ -149,13 +149,13 @@ and including the surrounding quotes."
 
 (add-json-encoder stringp jp-str)
 
-(define jp-maybe-string ((x vl-maybe-string-p) &key (ps 'ps))
+(define jp-maybe-string ((x maybe-stringp) &key (ps 'ps))
   :parents (json-encoders)
   (if x
       (jp-str x)
     (vl-print "null")))
 
-(add-json-encoder vl-maybe-string-p jp-maybe-string)
+(add-json-encoder maybe-stringp jp-maybe-string)
 
 
 
@@ -174,7 +174,7 @@ minimum reasonable size for a JSON implementation to support.</p>"
         ps))
     (vl-print-nat x)))
 
-(define jp-maybe-nat ((x vl-maybe-natp) &key (ps 'ps))
+(define jp-maybe-nat ((x maybe-natp) &key (ps 'ps))
   :parents (json-encoders)
   (if x
       (jp-nat x)
@@ -182,8 +182,8 @@ minimum reasonable size for a JSON implementation to support.</p>"
 
 (add-json-encoder natp jp-nat)
 (add-json-encoder posp jp-nat)
-(add-json-encoder vl-maybe-natp jp-maybe-nat)
-(add-json-encoder vl-maybe-posp jp-maybe-nat)
+(add-json-encoder maybe-natp jp-maybe-nat)
+(add-json-encoder maybe-posp jp-maybe-nat)
 
 
 
@@ -281,12 +281,12 @@ encoding.</p>"
 
 
 
-; Fancy automatic json encoding of cutil structures
+; Fancy automatic json encoding of std structures
 
 (program)
 
 (define make-json-encoder-alist
-  (efields   ;; A proper cutil::formallist-p for this structure's fields
+  (efields   ;; A proper std::formallist-p for this structure's fields
    omit      ;; A list of any fields to omit from the encoded output
    overrides ;; An alist of (fieldname . encoder-to-use-instead-of-default)
    world     ;; For looking up default encoders
@@ -294,7 +294,7 @@ encoding.</p>"
   ;; Returns an alist of the form (fieldname . encoder-to-use)
   (b* (((when (atom efields))
         nil)
-       ((cutil::formal e1) (car efields))
+       ((std::formal e1) (car efields))
        (- (cw "Determining encoder for ~x0... " e1.name))
 
        ;; Are we supposed to omit it?
@@ -328,7 +328,7 @@ encoding.</p>"
   (b* (((when (atom alist))
         nil)
        ((cons fieldname encoder) (car alist))
-       (foo->bar                 (cutil::da-accessor-name basename fieldname))
+       (foo->bar                 (std::da-accessor-name basename fieldname))
        ;; Suppose the fieldname is foo.  We create the string ["foo":].  We can
        ;; create this at macroexpansion time.
        (name1       (str::downcase-string (symbol-name fieldname)))
@@ -350,8 +350,8 @@ encoding.</p>"
        (elem-p         (mksym 'vl- type '-p))
        (elem-p-str     (symbol-name elem-p))
 
-       ((cutil::agginfo agg) (cutil::get-aggregate elem world))
-       ((unless (cutil::formallist-p agg.efields))
+       ((std::agginfo agg) (std::get-aggregate elem world))
+       ((unless (std::formallist-p agg.efields))
         (raise "Expected :efields for ~x0 to be a valid formallist, found ~x1."
                elem agg.efields))
        (enc-alist (make-json-encoder-alist agg.efields omit overrides world))
@@ -387,7 +387,7 @@ encoding.</p>"
 
 (defmacro def-vl-jp-aggregate (type &key omit override newlines
                                     (long '""))
-  (declare (xargs :guard (vl-maybe-natp newlines)))
+  (declare (xargs :guard (maybe-natp newlines)))
   `(make-event
     (let ((form (def-vl-jp-aggregate-fn ',type ',omit ',override ',long ',newlines
                   (w state))))
@@ -397,7 +397,7 @@ encoding.</p>"
 
 
 (defmacro def-vl-jp-list (type &key newlines)
-  (declare (xargs :guard (vl-maybe-natp newlines)))
+  (declare (xargs :guard (maybe-natp newlines)))
   (b* ((mksym-package-symbol 'vl::foo)
        (list-p         (mksym 'vl- type 'list-p))
        (elem-print     (mksym 'vl-jp- type))
