@@ -29,8 +29,8 @@
 ; about general concepts which the ordinary ACL2 user may have written
 ; other, conflicting theorems about.
 
-(include-book "cutil/top" :dir :system)
-(include-book "tools/defconsts" :dir :system)
+(include-book "std/util/top" :dir :system)
+(include-book "std/util/defconsts" :dir :system)
 (include-book "std/misc/two-nats-measure" :dir :system)
 (include-book "std/lists/list-defuns" :dir :system)
 (include-book "centaur/bitops/integer-length" :dir :system)
@@ -112,72 +112,6 @@ typical example is:</p>
           (mergesort x)))))
 
 
-(define vl-maybe-integerp (x)
-  :parents (utilities)
-  :short "Recognizer for integers and @('nil')."
-  :long "<p>This is like an option type, where either there is some integer
-value or nothing.</p>"
-  :inline t
-  (or (not x)
-      (integerp x))
-  ///
-  (defrule vl-maybe-integerp-compound-recognizer
-    (equal (vl-maybe-integerp x)
-           (or (integerp x)
-               (not x)))
-    :rule-classes :compound-recognizer))
-
-
-(define vl-maybe-natp (n)
-  :parents (utilities)
-  :short "Recognizer for naturals and @('nil')."
-  :long "<p>This is like an option type, where either there is some natural
-number value or nothing.</p>"
-  :inline t
-  (or (not n)
-      (natp n))
-  ///
-  (defrule vl-maybe-natp-compound-recognizer
-    (equal (vl-maybe-natp x)
-           (or (not x)
-               (natp x)))
-    :rule-classes :compound-recognizer))
-
-
-(define vl-maybe-posp (x)
-  :parents (utilities)
-  :short "Recognizer for positive naturals and @('nil')."
-  :long "<p>This is like an option type, where either there is some positive,
-natural number value or nothing.</p>"
-  :inline t
-  (or (not x)
-      (posp x))
-  ///
-  (defrule vl-maybe-posp-compound-recognizer
-    (equal (vl-maybe-posp x)
-           (or (and (integerp x)
-                    (< 0 x))
-               (not x)))
-    :rule-classes :compound-recognizer))
-
-
-(define vl-maybe-string-p (x)
-  ;; BOZO name consistency with other functions here
-  :parents (utilities)
-  :short "Recognizer for strings and @('nil')."
-  :long "<p>This is like an option type, where either there is some string
-value or nothing.</p>"
-  :inline t
-  (or (not x)
-      (stringp x))
-  ///
-  (defrule vl-maybe-string-p-compound-recognizer
-    (equal (vl-maybe-string-p x)
-           (or (not x)
-               (stringp x)))
-    :rule-classes :compound-recognizer))
-
-
 (deflist nat-listp (x)
   (natp x)
   :elementp-of-nil nil
@@ -185,7 +119,7 @@ value or nothing.</p>"
 
 
 (deflist vl-maybe-nat-listp (x)
-  (vl-maybe-natp x)
+  (maybe-natp x)
   :elementp-of-nil t
   :parents (utilities)
   :rest
@@ -196,7 +130,7 @@ value or nothing.</p>"
 
 
 (deflist vl-maybe-string-listp (x)
-  (vl-maybe-string-p x)
+  (maybe-stringp x)
   :elementp-of-nil t
   :parents (utilities)
   :rest
@@ -1164,3 +1098,20 @@ code that is violating guards), it just causes a hard error.</p>"
   (defun impossible ()
     (declare (xargs :guard nil))
     (er hard 'impossible "Provably impossible")))
+
+
+;; kind of a terrible function name
+(define vl-width-from-difference ((a integerp))
+  :returns (width posp :rule-classes :type-prescription)
+  (+ 1 (abs (lifix a))))
+
+
+(define anyp (x)
+  :parents (utilities)
+  :short "Recognizes any object."
+  :returns t
+  :long "<p>This just ignores @('x') and always returns @('t').  It is
+occasionally useful, e.g., when defining structures using macros that expect
+constraints for certain fields.</p>"
+  (declare (ignore x))
+  t)
