@@ -321,6 +321,11 @@
        :exec (and (integerp i)
                   (= i (- (ash 1 (integer-length i)) 1)))))
 
+(defund bitmaskp (i)
+  ;; replacement for logmaskp that respects int-equiv
+  (declare (xargs :guard (integerp i)))
+  (logmaskp (ifix i)))
+
 (defun-inline loghead (size i)
   ":doc-section logops-definitions
   (LOGHEAD size i) returns the size low-order bits of i.
@@ -348,10 +353,12 @@
 
 (defun logapp (size i j)
   ":doc-section logops-definitions
-  (LOGAPP size i j) is a binary append of i to j.
+  (LOGAPP size i j) is a binary append of i to j (where i effectively becomes
+  the 'low' bits and j becomes the 'high' bits).
   ~/~/
-  LOGAPP is a specification for merging integers.  Note that i is truncated
-  to size bits before merging with j.~/"
+  LOGAPP is a specification for merging integers.  Note that i is truncated to
+  size bits before merging with j, and that j is also shifted to the left by
+  size bits before the merge.~/"
   (declare (xargs :guard (and (integerp size)
                               (>= size 0)
                               (integerp i)
