@@ -19,9 +19,10 @@
 ; Original author: Jared Davis <jared@centtech.com>
 
 (in-package "STR")
-(include-book "cutil/define" :dir :system)
+(include-book "std/util/define" :dir :system)
 (include-book "natstr")
 (local (include-book "centaur/bitops/equal-by-logbitp" :dir :system))
+(local (include-book "centaur/bitops/ihsext-basics" :dir :system))
 (local (include-book "arithmetic/top-with-meta" :dir :system))
 (local (include-book "arithmetic"))
 
@@ -85,17 +86,17 @@ changed in future versions of the library.</p>
 
 <ul>
 
-<li><b>chars</b> refer to actual @(see characters), which typically will be in
-the Base64 alphabet, i.e., @('A-Z'), @('a-z'), @('0-9'), @('+'), and @('/').
-Of course, when decoding, data might be coming from some external source; it
-might contain pad characters @('=') or even arbitrary, invalid characters from
-beyond the Base64 alphabet.</li>
+<li><b>chars</b> refer to actual @(see acl2::characters), which typically will
+be in the Base64 alphabet, i.e., @('A-Z'), @('a-z'), @('0-9'), @('+'), and
+@('/').  Of course, when decoding, data might be coming from some external
+source; it might contain pad characters @('=') or even arbitrary, invalid
+characters from beyond the Base64 alphabet.</li>
 
 <li><b>codes</b> refer to actual @(see char-code)s, with the usual
 correspondence to <i>chars</i>.</li>
 
 <li><b>values</b> refer to the interpretation of each Base64 character as a
-numeric value.  For instance, the value of @('#\A') is 0; the value of @('#\B')
+numeric value.  For instance, the value of @('#\\A') is 0; the value of @('#\\B')
 is 1, and so on.  In our representation, the values are exactly the 64 natural
 numbers from 0-63, and can be recognized with, e.g., @('(unsigned-byte-p 6
 val)').</li>
@@ -1162,7 +1163,7 @@ an accumulator."
    (define base64-decode-easy ((x stringp))
      (b* (((mv ok orig) (base64-decode x))
           ((unless ok)
-           (cutil::raise "Whoops")
+           (raise "Whoops")
            ""))
        orig))
 
