@@ -20,11 +20,12 @@
 
 (in-package "STR")
 (include-book "strnatless")
+(include-book "std/basic/defs" :dir :system)
 (local (include-book "arithmetic"))
 (local (include-book "misc/assert" :dir :system))
 
 
-;; BOZO maybe use cutil::deflist for these list recognizers
+;; BOZO maybe use std::deflist for these list recognizers
 
 (defsection octal-digitp
   :parents (numbers)
