@@ -32,7 +32,7 @@
 (defttag :xdoc)
 
 (defun all-xdoc-topics (state)
-  (declare (xargs :mode :program))
+  (declare (xargs :stobjs state :verify-guards t))
   (prog2$
    (er hard? 'all-xdoc-topics "all-xdoc-topics not yet defined.")
    (mv-let (err val state)
