@@ -46,8 +46,10 @@
 ; nodes.
 
 (defconst *throwaway-tags*
-  (list "b" "i" "u" "tt" "color" "sf" "a" "box" "img"
-        "page" "long"))
+  ;; We leave "img" and "icon" in this list even thouggh we process them in
+  ;; merge-text below, because we don't want to process </img> or </icon>.
+  (list "b" "i" "u" "tt" "v" "color" "sf" "a" "box" "img" "icon"
+        "page"))
 
 (defun merge-text (x acc codes)
   ;; CODES is number of open <code> tags -- we don't normalize whitespace
@@ -61,7 +63,13 @@
              (codes (if (equal name "code")
                         (+ 1 codes)
                       codes)))
-          (cond ((member-equal name *throwaway-tags*)
+          (cond ((equal name "img")
+                 (b* ((tok  (list :TEXT "{IMAGE}")))
+                   (merge-text (cons tok rest) acc codes)))
+                ((equal name "icon")
+                 (b* ((tok  (list :TEXT "{ICON}")))
+                   (merge-text (cons tok rest) acc codes)))
+                ((member-equal name *throwaway-tags*)
                  (merge-text rest acc codes))
                 ((equal name "a")
                  (b* ((href (cdr (assoc :href (opentok-atts tok1))))
@@ -139,6 +147,8 @@
        ((when (equal name "code"))
         (+ 4 (get-indent-level (cdr open-tags))))
        ((when (equal name "blockquote"))
+        (+ 4 (get-indent-level (cdr open-tags))))
+       ((when (member-equal name '("table")))
         (+ 4 (get-indent-level (cdr open-tags)))))
     (get-indent-level (cdr open-tags))))
 
@@ -245,10 +255,10 @@
                          (t
                           list-nums)))
              (acc (cond
-                   ((equal name "parent")
-                    (b* ((acc (maybe-newline acc))
-                         (acc (str::revappend-chars "Parent topic: " acc)))
-                      acc))
+                   ;; ((equal name "parent")
+                   ;;  (b* ((acc (maybe-newline acc))
+                   ;;       (acc (str::revappend-chars "Parent: " acc)))
+                   ;;    acc))
                    ((equal name "li")
                     (b* ((bullet (if (eq (get-list-type open-tags) :bulleted)
                                      "* "
@@ -261,15 +271,17 @@
                          (acc     (append spaces acc))
                          (acc     (str::revappend-chars bullet acc)))
                       acc))
-                   ((member-equal name '("h4" "h5" "short" "p" "li" "dt" "dd" "br"
+                   ((member-equal name '("h4" "h5" "p" "li" "dt" "dd" "br"
                                          "index_head" "index_body" "blockquote"))
                     ;; This kind of tag has some level of indenting associated
                     ;; with it, so make sure we indent over to the right level.
                     (auto-indent (maybe-newline acc) open-tags))
-                   ((member-equal name '("code"))
+                   ((member-equal name '("code" "short" "long"))
                     (auto-indent (maybe-doublespace acc) open-tags))
                    ((member-equal name '("h1" "h2" "h3"))
                     (auto-indent (maybe-triplespace acc) open-tags))
+                   ((member-equal name '("table"))
+                    (auto-indent (maybe-newline acc) open-tags))
                    ((equal name "index")
                     (b* ((atts  (opentok-atts tok1))
                          (title (cdr (assoc-equal "title" atts)))
@@ -290,11 +302,13 @@
                           list-nums))
              (acc (cond
                    ((member-equal name '("h1" "h2" "h3" "h4" "h5" "p" "dl" "ul" "ol"
-                                         "short" "code" "parent" "index" "index_body"
-                                         "blockquote"))
+                                         "short" "code" "index" "index_body"
+                                         "blockquote" "table"))
                     (auto-indent (maybe-doublespace acc) open-tags))
-                   ((member-equal name '("li" "dd" "dt" "index_head"))
+                   ((member-equal name '("li" "dd" "dt" "index_head" "tr"))
                     (auto-indent (maybe-newline acc) open-tags))
+                   ((member-equal name '("td" "th"))
+                    (list* #\Space #\Space #\| #\Tab acc))
                    (t
                     acc))))
           (tokens-to-terminal rest wrap-col open-tags list-nums acc)))
@@ -325,7 +339,10 @@
   (b* ((name (cdr (assoc :name x)))
 ;       (- (cw "Preprocessing...~%"))
        ;; Use NIL as the topics-fal as a simple way to suppress autolinks...
-       ((mv text state) (preprocess-topic x all-topics nil nil state))
+       ((mv text state)
+        (preprocess-topic
+         (acons :parents nil x) ;; horrible hack so we can control this better
+         all-topics nil nil state))
 ;       (- (cw "Text is ~x0.~%" text))
 ;       (- (cw "Parsing xml...~%"))
        ((mv err tokens) (parse-xml text))
@@ -348,6 +365,15 @@
        (state (princ$ (symbol-package-name name) *standard-co* state))
        (state (princ$ "::" *standard-co* state))
        (state (princ$ (symbol-name name) *standard-co* state))
+       (from  (cdr (assoc :from x)))
+       (state (if from
+                  (princ$ (str::cat " -- " from) *standard-co* state)
+                state))
+       (parents (cdr (assoc :parents x)))
+       (state (if parents
+                  (fms "Parents: ~&0.~%" (list (cons #\0 parents))
+                       *standard-co* state nil)
+                (newline *standard-co* state)))
        (state (newline *standard-co* state))
        (state (princ$ terminal *standard-co* state))
        (state (newline *standard-co* state)))
@@ -372,7 +398,7 @@
 ;;          (cdr xml-tokens))
 ;;         (t
 ;;          (skip-through-close-long (cdr xml-tokens)))))
-    
+
 ;; (defun eliminate-long (xml-tokens)
 ;;   (cond ((atom xml-tokens)
 ;;          nil)
@@ -457,8 +483,8 @@
        (state (summarize-nearby-topics suggested-topics state))
        (state (newline *standard-co* state)))
     state))
-      
-             
+
+
 (defun colon-xdoc-fn (name all-topics state)
   (declare (xargs :guard (symbolp name)))
   (b* ((xdoc-entry (find-topic name all-topics))
@@ -466,7 +492,7 @@
        ((when (not xdoc-entry))
         (let ((state (suggest-alternatives name all-topics state)))
           (value :invisible)))
-       
+
        (state (display-topic xdoc-entry all-topics state)))
     (value :invisible)))
 
