@@ -24,28 +24,13 @@
 (in-package "XDOC")
 (include-book "autolink")
 (include-book "str/defs" :dir :system)
+(include-book "std/io/read-string" :dir :system)
+(include-book "unsound-eval")
+(include-book "verbosep")
 (local (include-book "misc/assert" :dir :system))
 (set-state-ok t)
 (program)
 
-(encapsulate
- ()
- ;; We're not allowed to attach to program mode functions without a ttag, so
- ;; make this logic mode.
- (logic)
- (defstub xdoc-verbose-p () t)
- (defun xdoc-verbose-fn ()
-   (declare (xargs :guard t))
-   t)
- (defun xdoc-quiet-fn ()
-   (declare (xargs :guard t))
-   nil)
- (defmacro xdoc-verbose ()
-   `(defattach xdoc-verbose-p xdoc-verbose-fn))
- (defmacro xdoc-quiet ()
-   `(defattach xdoc-verbose-p xdoc-quiet-fn))
- (xdoc-verbose))
-
 ; ----------------- World Lookup Stuff --------------------------
 
 (defun get-formals (fn world)
@@ -194,10 +179,32 @@
       (otherwise
        event))))
 
+#!ACL2
+(defun xdoc::get-event* (name wrld)
+  ;; Historically XDOC used ACL2::GET-EVENT, but we found that this was
+  ;; sometimes returning "bad" events like VERIFY-TERMINATION and
+  ;; VERIFY-TERMINATION-BOOT-STRAP instead of function definitions.  Matt
+  ;; Kaufmann developed this modification of ACL2::GET-EVENT to avoid these
+  ;; cases.
+  (let* ((wrld1 (lookup-world-index 'event
+                                    (getprop name 'absolute-event-number 0
+                                             'current-acl2-world wrld)
+                                    wrld))
+         (ev (access-event-tuple-form (cddr (car wrld1)))))
+    (cond ((null ev) nil)
+          ((and (consp ev)
+                (member-eq (car ev)
+                           '(verify-termination
+                             verify-termination-boot-strap
+                             verify-guards)))
+           (xdoc::get-event* name (cdr wrld1)))
+          (t ev))))
+
+
 (defun get-event-aux (name world)
   ;; A general purpose event lookup as in :pe
   (b* ((props (acl2::getprops name 'current-acl2-world world))
-       (evt   (and props (acl2::get-event name world)))
+       (evt   (and props (get-event* name world)))
        ((when evt)
         evt))
     (and (xdoc-verbose-p)
@@ -210,6 +217,21 @@
 (defun get-event (name world)
   (clean-up-event (get-event-aux name world)))
 
+(defun start-event (event acc)
+  (b* ((acc (str::revappend-chars "<b>" acc))
+       (acc (str::revappend-chars (case (and (consp event)
+                                             (car event))
+                                    (defun     "Function: ")
+                                    (defthm    "Theorem: ")
+                                    (defmacro  "Macro: ")
+                                    ;; not defstobj because it gets used for the
+                                    ;; stobjs and accessors...
+                                    (otherwise "Definition: "))
+                                  acc))
+       (acc (str::revappend-chars "</b>" acc)))
+    acc))
+
+
 #||
 
 (get-event-aux 'append (w state))
@@ -243,12 +265,6 @@
 
 ||#
 
-(defun get-def (fn world)
-  (get-event fn world))
-
-(defun get-theorem (name world)
-  ;; BOZO maybe do some cleaning to remove hints, etc.
-  (get-event name world))
 
 #||
 
@@ -327,12 +343,12 @@
 
 (defun read-through-some-char-aux (x n xl chars acc) ;; ==> (MV SUCCESSP STRING N-PRIME)
   (declare (type string x))
-  (if (eql xl n)
-      (mv nil (str::rchars-to-string acc) n)
-    (let ((charN (char x n)))
-      (if (member charN chars)
-          (mv t (str::rchars-to-string (cons charN acc)) n)
-        (read-through-some-char-aux x (+ 1 n) xl chars (cons charN acc))))))
+  (b* (((when (eql xl n))
+        (mv nil (str::rchars-to-string acc) n))
+       (charN (char x n))
+       ((when (member charN chars))
+        (mv t (str::rchars-to-string (cons charN acc)) n)))
+    (read-through-some-char-aux x (+ 1 n) xl chars (cons charN acc))))
 
 (defun read-through-some-char (x n xl chars)
   ;; Try to read until one of CHARS is found
@@ -348,53 +364,61 @@
         (t
          n)))
 
-(defun parse-directive (x n xl base-pkg kpa) ;; ==> (MV ERROR COMMAND ARG N-PRIME)
-  ;; Every directive has the form @(command arg)
+
+; Basic preprocessor directives:
+
+(defun parse-directive (x n xl base-pkg kpa) ;; ==> (MV ERROR COMMAND ARG ARG-RAW N-PRIME)
+  ;; Every basic directive has the form @(command arg)
   ;; Where command and arg are symbols.
   ;; We assume @( has just been read, and N is now pointing right after the open paren.
+  ;; We try to read the command and arg.
+  ;; Historically, on success, we returned the ARG as a symbol alone.
+  ;; Now, to support fancier @(see Foo) capitalization preservation, we need to also
+  ;; return the raw argument.
   (declare (type string x))
   (b* ((n                    (skip-past-ws x n xl))
        ;; We want to get nice error messages here, because if there'a a directive we
        ;; really do expect it to be a valid symbol.
        ((mv error command n) (parse-symbol x n xl (pkg-witness "XDOC") kpa t))
        ((when error)
-        (mv error nil nil n))
+        (mv error nil nil nil n))
        (n                    (skip-past-ws x n xl))
+       (arg-start-n          n)
        ((mv error arg n)     (parse-symbol x n xl base-pkg kpa t)))
       (cond
        ;; Some error parsing arg.  Add a little more context.
        (error (mv (str::cat "In " (symbol-name command) " directive: " error)
-                  nil nil n))
+                  nil nil nil n))
 
        ;; Ends with ), good.
        ((and (< n xl)
              (eql (char x n) #\)))
-        (mv nil command arg (+ n 1)))
+        (mv nil command arg (subseq x arg-start-n n) (+ n 1)))
 
        (t
         (mv (str::cat "In " (symbol-name command) " directive, expected ) after "
-                      (symbol-name arg)
-                      ". Near " (error-context x n xl) ".")
-            nil nil n)))))
+                      (symbol-name arg) ". Near " (error-context x n xl) ".")
+            nil nil nil n)))))
 
-;; (let ((x "body foo)"))
-;;   (parse-directive x 0 (length x) 'acl2::foo))
+#||
+(let ((x "body Foo)"))
+  (parse-directive x 0 (length x) 'acl2::foo (known-package-alist state)))
 
-;; (let ((x "body foo) bar"))
-;;   (parse-directive x 0 (length x) 'acl2::foo))
+(let ((x "body foo) bar"))
+  (parse-directive x 0 (length x) 'acl2::foo (known-package-alist state)))
 
-;; (let ((x "body xdoc::foo) bar"))
-;;   (parse-directive x 0 (length x) 'acl2::foo))
+(let ((x "body xdoc::foo) bar"))
+  (parse-directive x 0 (length x) 'acl2::foo (known-package-alist state)))
 
-;; (let ((x "xdoc::body xdoc::foo) bar"))
-;;   (parse-directive x 0 (length x) 'acl2::foo))
+(let ((x "xdoc::body xdoc::foo) bar"))
+  (parse-directive x 0 (length x) 'acl2::foo (known-package-alist state)))
 
-;; (let ((x "acl2::body xdoc::foo) bar"))
-;;   (parse-directive x 0 (length x) 'acl2::foo))
-
-;; (let ((x "acl2::body)xdoc::foo) bar"))
-;;   (parse-directive x 0 (length x) 'acl2::foo))
+(let ((x "acl2::body xdoc::foo) bar"))
+  (parse-directive x 0 (length x) 'acl2::foo (known-package-alist state)))
 
+(let ((x "acl2::body)xdoc::foo) bar"))
+  (parse-directive x 0 (length x) 'acl2::foo (known-package-alist state)))
+||#
 
 
 
@@ -423,30 +447,67 @@
   (b* ((acc            (sym-mangle-cap arg base-pkg acc)))
       (mv acc state)))
 
+(defun want-to-preserve-case-p (arg arg-raw base-pkg)
+  ;; Decide whether we want to preserve the case on a link like @(see Foo).
+  ;;  - ARG is the true symbol we're trying to link to.
+  ;;  - ARG-RAW is some string that the user typed to refer to ARG.
+  ;;  - BASE-PKG is the package we're printing relative to.
+  (and
+   ;; The symbol being linked to must be in the same package, because otherwise
+   ;; we're going to print a foo:: prefix and lowercasing everything seems
+   ;; quite reasonable.
+   (in-package-p arg base-pkg)
+   ;; You had better not have typed any sort of :: stuff, because if you did,
+   ;; then you wrote a link like @(see foo::Bar), and we are just going to
+   ;; downcase you.
+   (not (position #\: arg-raw))
+   ;; You had better have at least one lowercase letter somewhere, because
+   ;; otherwise it's likely that you're a macro that wrote @(see GUARD) and we
+   ;; don't want to let you shout at us.
+   (str::string-has-some-down-alpha-p arg-raw 0 (length arg-raw))
+   ;; We'll require some upper case character too, because otherwise we may as
+   ;; well use our default mechanism and capitalize, e.g., ACL2.
+   (str::string-has-some-up-alpha-p arg-raw 0 (length arg-raw))
+   ;; For good measure, let's make sure you are really naming our symbol.  This
+   ;; should hopefully prevent any kind of mistakes from symbols with bars or
+   ;; backslash escape characters.
+   (str::istreqv arg-raw (symbol-name arg))))
 
-(defun process-see-directive (arg base-pkg state acc) ;; ===> (MV ACC STATE)
+(defun process-see-directive (arg arg-raw base-pkg state acc) ;; ===> (MV ACC STATE)
 
-; @(see foo) just expands into a link with a lowercase name.
-
-  (b* ((acc            (str::revappend-chars "<see topic=\"" acc))
-       (acc            (file-name-mangle arg acc))
-       (acc            (str::revappend-chars "\">" acc))
-       (acc            (sym-mangle arg base-pkg acc))
-       (acc            (str::revappend-chars "</see>" acc)))
+; @(see foo) just expands into a link with a (usually) lowercase name, but we go to
+; some trouble to preserve case for things like @(see Guard).
+ 
+  (b* ((acc (str::revappend-chars "<see topic=\"" acc))
+       (acc (file-name-mangle arg acc))
+       (acc (str::revappend-chars "\">" acc))
+       (acc (if (want-to-preserve-case-p arg arg-raw base-pkg)
+                (str::revappend-chars (str::trim arg-raw) acc)
+              (sym-mangle arg base-pkg acc)))
+       (acc (str::revappend-chars "</see>" acc)))
       (mv acc state)))
 
-
 (defun process-see-cap-directive (arg base-pkg state acc) ;; ===> (MV ACC STATE)
 
 ; @(csee foo) just expands into a link with a capitalized name.
 
-  (b* ((acc            (str::revappend-chars "<see topic=\"" acc))
-       (acc            (file-name-mangle arg acc))
-       (acc            (str::revappend-chars "\">" acc))
-       (acc            (sym-mangle-cap arg base-pkg acc))
-       (acc            (str::revappend-chars "</see>" acc)))
+  (b* ((acc (str::revappend-chars "<see topic=\"" acc))
+       (acc (file-name-mangle arg acc))
+       (acc (str::revappend-chars "\">" acc))
+       (acc (sym-mangle-cap arg base-pkg acc))
+       (acc (str::revappend-chars "</see>" acc)))
       (mv acc state)))
 
+(defun process-tsee-directive (arg arg-raw base-pkg state acc) ;; ===> (MV ACC STATE)
+
+; @(tsee foo) is basically <tt>@(see ...)</tt>.
+
+  (b* ((acc            (str::revappend-chars "<tt>" acc))
+       ((mv acc state) (process-see-directive arg arg-raw base-pkg state acc))
+       (acc            (str::revappend-chars "</tt>" acc)))
+    (mv acc state)))
+
+
 
 ;; (defconst *xdoc-link-file-message*
 ;;   "; This is an XDOC Link file.
@@ -553,8 +614,9 @@
 ; @(def foo) -- look up the definition for foo, pretty-print it in a <code>
 ; block, along with a source-code link.
 
-  (b* ((def            (get-def arg (w state)))
-       (acc            (str::revappend-chars "<p><b>Definition: </b>" acc))
+  (b* ((def            (get-event arg (w state)))
+       (acc            (str::revappend-chars "<p>" acc))
+       (acc            (start-event def acc))
        ((mv acc state) (process-srclink-directive arg dir state acc))
        (acc            (str::revappend-chars "</p>" acc))
        (acc            (str::revappend-chars "<code>" acc))
@@ -568,8 +630,9 @@
 ; but don't use a source-code link because this is a "Generated Definition" for
 ; which a tags-search will probably fail.
 
-  (b* ((def            (get-def arg (w state)))
-       (acc            (str::revappend-chars "<p><b>Definition: </b>" acc))
+  (b* ((def            (get-event arg (w state)))
+       (acc            (str::revappend-chars "<p>" acc))
+       (acc            (start-event def acc))
        (acc            (sym-mangle arg base-pkg acc))
        (acc            (str::revappend-chars "</p>" acc))
        (acc            (str::revappend-chars "<code>" acc))
@@ -582,12 +645,13 @@
 ; @(thm foo) -- Look up the theorem named foo, and pretty-print its event along
 ; with a source link.
 
-  (b* ((theorem        (get-theorem arg (w state)))
-       (acc            (str::revappend-chars "<p><b>Theorem: </b>" acc))
+  (b* ((def            (get-event arg (w state)))
+       (acc            (str::revappend-chars "<p>" acc))
+       (acc            (start-event def acc))
        ((mv acc state) (process-srclink-directive arg dir state acc))
        (acc            (str::revappend-chars "</p>" acc))
        (acc            (str::revappend-chars "<code>" acc))
-       ((mv acc state) (xml-ppr-obj-aux theorem topics-fal base-pkg state acc))
+       ((mv acc state) (xml-ppr-obj-aux def topics-fal base-pkg state acc))
        (acc            (str::revappend-chars "</code>" acc)))
       (mv acc state)))
 
@@ -596,12 +660,13 @@
 ; @(gthm foo) -- Like @(thm foo), but don't provide a source link since this is
 ; a generated theorem.
 
-  (b* ((theorem        (get-theorem arg (w state)))
-       (acc            (str::revappend-chars "<p><b>Theorem: </b>" acc))
+  (b* ((def            (get-event arg (w state)))
+       (acc            (str::revappend-chars "<p>" acc))
+       (acc            (start-event def acc))
        (acc            (sym-mangle arg base-pkg acc))
        (acc            (str::revappend-chars "</p>" acc))
        (acc            (str::revappend-chars "<code>" acc))
-       ((mv acc state) (xml-ppr-obj-aux theorem topics-fal base-pkg state acc))
+       ((mv acc state) (xml-ppr-obj-aux def topics-fal base-pkg state acc))
        (acc            (str::revappend-chars "</code>" acc)))
       (mv acc state)))
 
@@ -648,7 +713,7 @@
       (mv acc state)))
 
 
-(defun process-directive (command arg dir topics-fal base-pkg state acc) ;; ===> (MV ACC STATE)
+(defun process-directive (command arg arg-raw dir topics-fal base-pkg state acc) ;; ===> (MV ACC STATE)
 
 ; Command and Arg are the already-parsed symbols we have read from the
 ; documentation string.  Carry out whatever directive we've been asked to do.
@@ -666,8 +731,9 @@
     (call      (process-call-directive    arg base-pkg state acc))
     (ccall     (process-ccall-directive   arg base-pkg state acc))
     (url       (process-url-directive     arg state acc))
-    (see       (process-see-directive     arg base-pkg state acc))
+    (see       (process-see-directive     arg arg-raw base-pkg state acc))
     (csee      (process-see-cap-directive arg base-pkg state acc))
+    (tsee      (process-tsee-directive    arg arg-raw base-pkg state acc))
     (sym       (process-sym-directive     arg base-pkg state acc))
     (csym      (process-sym-cap-directive arg base-pkg state acc))
     (otherwise
@@ -718,6 +784,25 @@
 ;
 ; To fix this, we look at the text between <code> and </code>, and if every
 ; line begins with a space, we eat those spaces.
+;
+; PATCH.  I eventually found that I wanted to be even more permissive than
+; the above.  Sometimes when I write preprocessor things in @({...}) blocks,
+; I find that I want to separate what I'm writing from the markup even more.
+; That is, it's sort of easier to ignore the markup when I write:
+;
+;  |@({
+;  |    this is some stuff
+;  |})
+;
+; Than:
+;
+;  |@({
+;  | this is some stuff
+;  |})
+;
+; At any rate, it seems pretty reasonable to try to eat up any amount of spaces
+; that are at the start of every line in the block.  So, now, instead of just
+; eating one space, we eat however many spaces are shared by all lines.
 
 (encapsulate
   ()
@@ -742,58 +827,117 @@
               (eql (fifth acc) #\c)
               (eql (sixth acc) #\<)))))
 
-(defun read-code-segment (x n xl acc always-spacep)
-  "Returns (MV N ACC ALWAYS-SPACEP)"
+(defun read-code-segment
 
 ; We assume we're inside a <code> block.  We read until </code> is found,
-; gathering the characters we see and tracking whether each newline is followed
-; by a space.
+; gathering the characters we see and tracking how many spaces follow each
+; newline.
 
+  (x n xl    ; string we're reading, position, length as usual
+     acc     ; accumulator for chars in this <code> block so far
+     spacesp ; are we currently reading spaces?
+     curr-sp ; how many spaces we've seen on this line so far
+     min-sp  ; minimum number of spaces we've seen on any prev line
+     )
+  "Returns (MV N ACC MIN-SP)"
   (b* (((when (>= n xl))
-        (mv n acc always-spacep))
+        ;; End of string while reading <code>?  Shouldn't really happen.
+        ;; Let's return 0 so that we won't change the section.
+        (mv n acc 0))
 
        (char-n (char x n))
        ((when (and (eql char-n #\<)
                    (str::strprefixp-impl "</code>" x 0 n
                                          7 ;; (length "</code>")
                                          xl)))
-        ;; Found </code> tag, so stop reading
-        (mv n acc always-spacep))
+        ;; Found </code> tag, so stop reading.  This is kind of subtle.
+        ;;
+        ;; Case 1: we had </code> at the start of the line, or after just some
+        ;; spaces.  In this case, we don't care how many spaces we've seen so
+        ;; far, because we explicitly want to allow for things like
+        ;;
+        ;;   |<code>
+        ;;   |  foo
+        ;;   |</code>
+        ;;
+        ;; Case 2: someone put the </code> inline with their text, i.e., they
+        ;; wrote something like
+        ;;
+        ;;   |<code>
+        ;;   |  foo
+        ;;   | bar</code>
+        ;;
+        ;; In this case, we definitely DO want to consider spaces on the
+        ;; current line.
+        (mv n acc (if spacesp
+                      min-sp
+                    (min curr-sp min-sp))))
 
        (acc (cons char-n acc))
+       ((when (eql char-n #\Space))
+        (read-code-segment x (+ 1 n) xl
+                           acc
+                           spacesp
+                           (if spacesp (+ 1 curr-sp) curr-sp)
+                           min-sp))
+
        ((unless (eql char-n #\Newline))
         ;; A normal character, just accumulate it.
-        (read-code-segment x (+ 1 n) xl acc always-spacep))
+        (read-code-segment x (+ 1 n) xl
+                           acc nil curr-sp min-sp)))
 
-       (n (+ 1 n))
-       ((when (>= n xl))
-        ;; End of string while reading <code>?  Shouldn't really happen...
-        (mv n acc always-spacep))
+    ;; For newlines, the situation is also subtle.  We want to allow
+    ;; people to double-space things, e.g.,
+    ;;
+    ;;    |<code>
+    ;;    |  foo
+    ;;    |[newline here]
+    ;;    |  bar
+    ;;    |</code>
+    ;;
+    ;; So basically: consider curr-sp only if there are non-spaces here.
+    (read-code-segment x (+ 1 n) xl
+                       acc
+                       t ;; start reading spaces again
+                       0 ;; new line has no spaces yet
+                       (if spacesp
+                           min-sp
+                         (min curr-sp min-sp)))))
 
-       (char-n (char x n))
-       ((when (eql char-n #\<))
-        ;; We allow the </code> to come without a space.
-        (mv n acc always-spacep))
+#||
+ (b* ((x  "      three spaces
+  two spaces
+ blah</code> more text <p>blah blah</p>")
+     ((mv n acc min-sp)
+      (read-code-segment x 0 (length x) nil t 0 100)))
+  (list n (str::rchars-to-string acc) min-sp))
+||#
 
-       (acc (cons char-n acc))
-       (always-spacep (and always-spacep
-                           (or (eql char-n #\Space)
-                               (eql char-n #\Newline)))))
-    (read-code-segment x (+ 1 n) xl acc always-spacep)))
+(defun starts-with-n-spaces (n x)
+  (or (zp n)
+      (and (consp x)
+           (eql (car x) #\Space)
+           (starts-with-n-spaces (- n 1) (cdr x)))))
 
-(defun revappend-code-chars (code-chars acc always-spacep)
+(defun revappend-code-chars
+  (code-chars ; characters from the <code> section (regular, non-reversed order)
+   acc        ; accumulator we're writing them into
+   min-sp     ; minimum spaces after each newline (except blank lines)
+   )
   (b* (((when (atom code-chars))
         acc)
-       (char1 (car code-chars))
-       (acc   (cons char1 acc))
+       ((cons char1 rest) code-chars)
+       (acc (cons char1 acc))
        ((unless (eql char1 #\Newline))
-        (revappend-code-chars (cdr code-chars) acc always-spacep))
-       ((when (and always-spacep
-                   (consp (cdr code-chars))
-                   (eql (second code-chars) #\Space)))
-        ;; Skip the first space
-        (revappend-code-chars (cddr code-chars) acc always-spacep)))
-    (revappend-code-chars (cdr code-chars) acc always-spacep)))
+        (revappend-code-chars (cdr code-chars) acc min-sp))
+       ;; Have a newline.
+       ((when (and (posp min-sp)
+                   (starts-with-n-spaces min-sp rest)))
+        ;; Skip these n spaces
+        (revappend-code-chars (nthcdr min-sp rest) acc min-sp)))
+    ;; Else, doesn't start with n spaces.  Must be a pure blank line.
+    ;; Don't muck with it.
+    (revappend-code-chars rest acc min-sp)))
 
 (defun transform-code-segments (x n xl acc)
   (b* (((when (>= n xl))
@@ -805,108 +949,85 @@
                      (just-started-code-p acc)))
         (transform-code-segments x n xl acc))
        ;; Started a code segment.
-       ((mv n code-acc always-spacep)
-        (read-code-segment x n xl nil t))
+       ((mv n code-acc min-sp)
+        (read-code-segment x n xl
+                           nil ;; accumulator for chars
+                           t   ;; yes, we're reading spaces, to begin
+                           0   ;; no spaces seen so far
+                           100 ;; as good as infinity, here.
+                           ))
        (code-chars (reverse code-acc))
-       (acc (revappend-code-chars code-chars acc always-spacep)))
+       (acc (revappend-code-chars code-chars acc min-sp)))
     (transform-code-segments x n xl acc)))
 
 (defun transform-code (x)
   ;; Fix leading spaces in <code> segments
   (str::rchars-to-string (transform-code-segments x 0 (length x) nil)))
 
-;; (transform-code 
-;; "<p>This is 
-;; some regular text</p>
-;; <code>
-;;     blah1
-;;     <blah>blah2</blah>
-;;     blah3
-;; </code>
-;; <p>And more text</p>")
+#||
+ (transform-code 
+"<p>This is 
+some regular text</p>
+<code>     
+    blah1
+    <blah>blah2</blah>
+    blah3
+</code>
+<p>And more text</p>")
+||#
 
-(defun advance-line (x n xl)
-  ;; Returns the index of the next newline after character N, or else XL
-  (b* (((when (eql n xl))
-        n)
-       (c (char x n))
-       ((when (eql c #\Newline))
-        n))
-    (advance-line x (+ 1 n) xl)))
 
-(defun all-lines-start-with-space-p (x n xl)
-  ;; Assumptions:
-  ;;  - We are starting a line
-  ;;  - Every line we've seen so far starts with a space
+
+
+; Horribly, we now pretty much repeat the same thing, but whereas the above
+; targets <code> blocks, we now deal with @({...}) blocks.  Here, we assume
+; that we've already extracted the whole string that we want to transform.
+
+(defun read-ppcode-segment
+  (x n xl    ; string we're reading, position, length as usual
+     spacesp ; are we currently reading spaces?
+     curr-sp ; how many spaces we've seen on this line so far
+     min-sp  ; minimum number of spaces we've seen on any prev line
+     )
+  "Returns MIN-SP"
   (b* (((when (>= n xl))
-        ;; DO NOT WEAKEN TO (= N XL).  We unusually allow this to exceed the
-        ;; string length; consider the call of advance-line below.
-        ;; Success, reached end of string, no space required
-        t)
-       (c (char x n))
-       ((when (eql c #\Newline))
-        ;; Fine, blank line, no space required
-        (all-lines-start-with-space-p x (+ 1 n) xl))
-       ((when (eql c #\Space))
-        ;; Fine, has a space, skip to next line
-        (b* ((n (advance-line x (+ 1 n) xl)))
-          (all-lines-start-with-space-p x (+ 1 n) xl))))
-    ;; Nope, found a non-space char at start of the line.
-    nil))
+        ;; End of string.  That's when we want to stop.
+        (if spacesp
+            min-sp
+          (min curr-sp min-sp)))
 
-(encapsulate
-  ()
-  (logic)
+       (char-n (char x n))
+       ;; No </code> handling; we'll read to the end of the string.
+       ((when (eql char-n #\Space))
+        (read-ppcode-segment x (+ 1 n) xl
+                             spacesp
+                             (if spacesp (+ 1 curr-sp) curr-sp)
+                             min-sp))
 
-  (local
-   (progn
-
-     (defun test (x)
-       (declare (xargs :mode :program))
-       (all-lines-start-with-space-p x 0 (length x)))
-
-     (assert! (test " foo
- bar
- baz"))
-
-     (assert! (not (test " foo
-bar
- baz")))
-
-     (assert! (test " foo
-
- bar
- baz
-"))
-
-     (assert! (not (test "foo
- bar
-baz
-"))))))
-
-(defun delete-leading-line-space (x n xl acc)
-  ;; Assumes (all-lines-start-with-space-p x n xl)
-  (b* (((when (>= n xl))
-        acc)
-       (c (char x n))
-       ((when (eql c #\Newline))
-        (delete-leading-line-space x (+ 1 n) xl (cons c acc)))
-       ;; Else it must be a space.  Skip the space but copy the other chars
-       ;; through the end of the line.
-       (start (+ 1 n))
-       (end   (advance-line x start xl))
-       ;; BOZO abuse of revappend-chars-aux to only copy until end...
-       ;; should fix its guard to allow this sort of thing
-       (acc   (str::revappend-chars-aux x start end acc)))
-    (delete-leading-line-space x end xl acc)))
+       ((unless (eql char-n #\Newline))
+        ;; normal character, stop reading spaces if we were doing so
+        (read-ppcode-segment x (+ 1 n) xl
+                             nil curr-sp min-sp)))
+    ;; Newline character, start new line
+    (read-ppcode-segment x (+ 1 n) xl
+                         t ;; start reading spaces again
+                         0 ;; new line has no spaces yet
+                         (if spacesp
+                             min-sp
+                           (min curr-sp min-sp)))))
 
 (defun maybe-fix-spaces-in-sub (x)
   (declare (type string x))
-  (b* ((xl (length x))
-       ((unless (all-lines-start-with-space-p x 0 xl))
+  (b* ((xl      (length x))
+       (min-sp  (read-ppcode-segment x 0 xl t 0 100))
+       ((when (zp min-sp))
         ;; Fine, don't do anything
         x)
-       (acc (delete-leading-line-space x 0 xl nil)))
+       (code-chars (explode x))
+       (code-chars (if (starts-with-n-spaces min-sp code-chars)
+                       (nthcdr min-sp code-chars)
+                     code-chars))
+       (acc (revappend-code-chars code-chars nil min-sp)))
     (str::rchars-to-string acc)))
 
 (encapsulate
@@ -988,8 +1109,112 @@ baz
     (subseq x start end)))
 
 
+; Support for Lisp Evaluation
+;
+; @(`(foo ...)`) evaluates foo and pretty-prints the result with <tt>s
+;
+; @(`(:code (foo ...))`) evaluates foo with <code>s
+;
+; @(`(:raw (foo ...))`) evaluates foo, which must return STR or (mv STR ...),
+; and inserts STR into the accumulator, verbatim, without any special
+; encoding.
+;
+; We might add other keywords later.
 
+(defun preprocess-eval-parse
+  (str       ;; the ... part of the @(`...`) form
+   base-pkg  ;; base package for this topic, which we need to parse relative to
+   state)
+  "Returns (mv errmsg? parsed-sexpr state)"
+  (b* ((curr-pkg         (current-package state))
+       ;; Switch to the base-pkg to do the parsing, so that we read the symbols
+       ;; in the string relative to the base-pkg that was used at the time...
+       ((mv err & state) (acl2::in-package-fn (symbol-package-name base-pkg) state))
+       ((when err)
+        (mv (msg "Error: can't switch to package ~s0 to parse @(`...`).  Input: ~s1"
+                 (symbol-package-name base-pkg) str)
+            nil
+            state))
+       ((mv err objects state) (acl2::read-string str))
+       ((mv & & state) (acl2::in-package-fn curr-pkg state))
+       ((when err)
+        (mv (msg "Error: failed to parse @(`...`): ~@0.  Input: ~s1" err str)
+            nil
+            state))
+       ((unless (equal (len objects) 1))
+        (mv (msg "Error: parsed ~x0 expressions (instead of 1) from @(`...`).  Input: ~s1."
+                 (len objects) str)
+            nil
+            state))
+       (sexpr (car objects)))
+    (mv nil sexpr state)))
 
+(defun preprocess-eval-main (sexpr topics-fal base-pkg kpa state acc)
+  "Returns (MV ERRMSG? ACC STATE)"
+  (declare (ignorable kpa))
+  (b* (((mv kind sexpr)
+        (if (and (consp sexpr)
+                 (consp (cdr sexpr))
+                 (not (cddr sexpr))
+                 (keywordp (first sexpr)))
+            (mv (first sexpr) (second sexpr))
+          (mv nil sexpr)))
+       ((mv err vals state) (acl2::unsound-eval sexpr))
+       ((when err)
+        (mv (msg "Error: failed to evaluate @(`...`): ~@0.  Input expression: ~x1" err sexpr)
+            acc state))
+       (ret (cond ((atom vals)
+                   ;; The use of ER here is special; this is just a sanity
+                   ;; check, this should be an impossible case, not triggerable
+                   ;; by the user, just due to how unsound-eval works.
+                   (er hard? 'preprocess-eval-main "unsound-eval returned an atom?"))
+                  ((atom (cdr vals))
+                   (car vals))
+                  (t
+                   (cons 'mv vals))))
+
+       ((unless kind)
+        (b* ((acc            (str::revappend-chars "<v>" acc))
+             ((mv acc state) (xml-ppr-obj-aux ret topics-fal base-pkg state acc))
+             (acc            (str::revappend-chars "</v>" acc)))
+          (mv nil acc state)))
+
+       ((when (eq kind :code))
+        (b* ((acc            (str::revappend-chars "<code>" acc))
+             ((mv acc state) (xml-ppr-obj-aux ret topics-fal base-pkg state acc))
+             (acc            (str::revappend-chars "</code>" acc)))
+          (mv nil acc state)))
+
+       ((when (eq kind :raw))
+        (b* ((str (car vals))
+             ((unless (stringp str))
+              (mv (msg "Error: @(`(:raw ...)`) must return a string as its first (or only) ~
+                        return value, but ~x0 returned ~x1."
+                       sexpr ret)
+                  acc state))
+             (acc (str::revappend-chars str acc)))
+          (mv nil acc state))))
+    (mv (msg "Error: @(`(~x0 ...)`) is not a known evaluation keyword.  Input sexpr: ~x1."
+             kind sexpr)
+        acc state)))
+
+(defun preprocess-eval (str topics-fal base-pkg kpa state acc)
+  "Returns (MV ACC STATE)"
+  (b* (((mv errmsg sexpr state) (preprocess-eval-parse str base-pkg state))
+       ((when errmsg)
+        (or (not (xdoc-verbose-p))
+            (cw "; xdoc error: ~@0~%" errmsg))
+        (fmt-and-encode-to-acc-aux "[~@0]" (list (cons #\0 errmsg))
+                                   base-pkg state acc))
+
+       ((mv errmsg acc state)
+        (preprocess-eval-main sexpr topics-fal base-pkg kpa state acc))
+       ((when errmsg)
+        (or (not (xdoc-verbose-p))
+            (cw "; xdoc error: ~@0~%" errmsg))
+        (fmt-and-encode-to-acc-aux "[~@0]" (list (cons #\0 errmsg))
+                                   base-pkg state acc)))
+    (mv acc state)))
 
 (defun preprocess-aux (x n xl dir topics-fal base-pkg kpa state acc)
   "Returns (MV ACC STATE)"
@@ -1020,9 +1245,9 @@ baz
                                         (cw "; xdoc error: no closing ') found for @(' ...~%"))
                                    (mv acc state)))
                           (sub (fancy-extract-block x (+ n 3) end))
-                          (acc (str::revappend-chars "<tt>" acc))
+                          (acc (str::revappend-chars "<v>" acc))
                           (acc (autolink-and-encode sub 0 (length sub) topics-fal base-pkg kpa acc))
-                          (acc (str::revappend-chars "</tt>" acc)))
+                          (acc (str::revappend-chars "</v>" acc)))
                        (preprocess-aux x (+ end 2) xl dir topics-fal base-pkg kpa state acc)))
 
                     ((when (and (< (+ n 2) xl)
@@ -1039,14 +1264,25 @@ baz
                           (acc (str::revappend-chars "</code>" acc)))
                        (preprocess-aux x (+ end 2) xl dir topics-fal base-pkg kpa state acc)))
 
+                    ((when (and (< (+ n 2) xl)
+                                (eql (char x (+ n 2)) #\`)))
+                     ;; @(`...`) directive -- Lisp evaluation of the form.
+                     (b* ((end (str::strpos-fast "`)" x (+ n 2) 2 xl))
+                          ((unless end)
+                           (prog2$ (and (xdoc-verbose-p)
+                                        (cw "; xdoc error: no closing `) found for @(` ...~%"))
+                                   (mv acc state)))
+                          (str (subseq x (+ n 3) end))
+                          ((mv acc state) (preprocess-eval str topics-fal base-pkg kpa state acc)))
+                       (preprocess-aux x (+ end 2) xl dir topics-fal base-pkg kpa state acc)))
 
-                    ((mv error command arg n) (parse-directive x (+ n 2) xl base-pkg kpa))
+                    ((mv error command arg arg-raw n) (parse-directive x (+ n 2) xl base-pkg kpa))
                     ((when error)
                      (prog2$ (and (xdoc-verbose-p)
                                   (cw "; xdoc error: ~x0.~%" error))
                              (mv acc state)))
                     ((mv acc state)
-                     (process-directive command arg dir topics-fal base-pkg state acc)))
+                     (process-directive command arg arg-raw dir topics-fal base-pkg state acc)))
                  (preprocess-aux x n xl dir topics-fal base-pkg kpa state acc)))
 
               (t
