@@ -25,14 +25,11 @@
 (in-package "XDOC")
 (include-book "save-classic")
 (include-book "importance")
+(include-book "linkcheck")
 (include-book "centaur/bridge/to-json" :dir :system)
 (set-state-ok t)
 (program)
 
-
-
-
-
 (defun-inline json-encode-string (x acc)
   (declare (type string x))
   (cons #\" (bridge::json-encode-str x (cons #\" acc))))
@@ -169,6 +166,65 @@
 ; viz., their position in the array.  This seems like a useful capability for
 ; implementing search features.
 
+(defun short-xml-for-topic (topic topics-fal state)
+  ;; Returns (mv xml-string state)
+  (b* ((short    (or (cdr (assoc :short topic)) ""))
+       (base-pkg (cdr (assoc :base-pkg topic)))
+       (name     (cdr (assoc :name topic)))
+       ((mv short-rchars state) (preprocess-main short nil topics-fal base-pkg state nil))
+       (short-str (str::rchars-to-string short-rchars))
+       ((mv err &) (parse-xml short-str))
+       (state
+        (if err
+            (pprogn
+               (fms "~|~%WARNING: problem with :short in topic ~x0:~%"
+                    (list (cons #\0 name))
+                    *standard-co* state nil)
+               (princ$ err *standard-co* state)
+               (fms "~%~%" nil *standard-co* state nil))
+          state))
+       (short-str-xml
+        (b* (((unless err)
+              short-str)
+             (tmp nil)
+             (tmp (str::revappend-chars
+                   "<b><color rgb='#ff0000'>Markup error in :short</color></b>
+                    <code>" tmp))
+             (tmp (simple-html-encode-str err 0 (length err) tmp))
+             (tmp (str::revappend-chars "</code>" tmp)))
+          (str::rchars-to-string tmp))))
+    (mv short-str-xml state)))
+
+(defun long-xml-for-topic (topic topics-fal state)
+  ;; Returns (mv xml-string state)
+  (b* ((long     (or (cdr (assoc :long topic)) ""))
+       (base-pkg (cdr (assoc :base-pkg topic)))
+       (name     (cdr (assoc :name topic)))
+       ((mv long-rchars state) (preprocess-main long nil topics-fal base-pkg state nil))
+       (long-str (str::rchars-to-string long-rchars))
+       ((mv err &) (parse-xml long-str))
+       (state
+        (if err
+            (pprogn
+               (fms "~|~%WARNING: problem with :long in topic ~x0:~%"
+                    (list (cons #\0 name))
+                    *standard-co* state nil)
+               (princ$ err *standard-co* state)
+               (fms "~%~%" nil *standard-co* state nil))
+          state))
+       (long-str-xml
+        (b* (((unless err)
+              long-str)
+             (tmp nil)
+             (tmp (str::revappend-chars
+                   "<h3><color rgb='#ff0000'>Markup error in :long</color></h3>
+                    <code>" tmp))
+             (tmp (simple-html-encode-str err 0 (length err) tmp))
+             (tmp (str::revappend-chars "</code>" tmp)))
+          (str::rchars-to-string tmp))))
+    (mv long-str-xml state)))
+
+
 (defun json-encode-index-entry (topic topics-fal uid-map state acc)
   (b* ((name     (cdr (assoc :name topic)))
        (base-pkg (cdr (assoc :base-pkg topic)))
@@ -193,29 +249,7 @@
 
        (parent-uids (collect-uids parents uid-map))
 
-       ((mv short-rchars state) (preprocess-main short nil topics-fal base-pkg state nil))
-       (short-str (str::rchars-to-string short-rchars))
-       ((mv err &) (parse-xml short-str))
-       (state
-        (if err
-            (pprogn
-               (fms "~|~%WARNING: problem with :short in topic ~x0:~%"
-                    (list (cons #\0 name))
-                    *standard-co* state nil)
-               (princ$ err *standard-co* state)
-               (fms "~%~%" nil *standard-co* state nil))
-          state))
-
-       (short-str
-        (b* (((unless err)
-              short-str)
-             (tmp nil)
-             (tmp (str::revappend-chars
-                   "<b><color rgb='#ff0000'>Markup error in :short</color></b>
-                    <code>" tmp))
-             (tmp (simple-html-encode-str err 0 (length err) tmp))
-             (tmp (str::revappend-chars "</code>" tmp)))
-          (str::rchars-to-string tmp)))
+       ((mv short-str state) (short-xml-for-topic topic topics-fal state))
 
 ; I originally used a JSON object like {"name":"Append","rawname":"..."}  But
 ; then some back-of-the-napkin calculations said that these nice names were
@@ -250,8 +284,9 @@
 #||
 (b* ((topics (get-xdoc-table (w state)))
      (topics-fal (topics-fal topics))
+     (uid-map nil)
      ((mv acc state)
-      (json-encode-index-entry (car topics) topics-fal state nil))
+      (json-encode-index-entry (car topics) topics-fal uid-map state nil))
      (state (princ$ (str::rchars-to-string acc) *standard-co* state)))
   state)
 ||#
@@ -309,28 +344,7 @@
         (mv (er hard? __function__ "From is not a string: ~x0.~%" topic)
             state))
 
-       ((mv long-rchars state) (preprocess-main long nil topics-fal base-pkg state nil))
-       (long-str (str::rchars-to-string long-rchars))
-       ((mv err &) (parse-xml long-str))
-       (state
-        (if err
-            (pprogn
-               (fms "~|~%WARNING: problem with :long in topic ~x0:~%"
-                    (list (cons #\0 name))
-                    *standard-co* state nil)
-               (princ$ err *standard-co* state)
-               (fms "~%~%" nil *standard-co* state nil))
-          state))
-       (long-str
-        (b* (((unless err)
-              long-str)
-             (tmp nil)
-             (tmp (str::revappend-chars
-                   "<h3><color rgb='#ff0000'>Markup error in :long</color></h3>
-                    <code>" tmp))
-             (tmp (simple-html-encode-str err 0 (length err) tmp))
-             (tmp (str::revappend-chars "</code>" tmp)))
-          (str::rchars-to-string tmp)))
+       ((mv long-str state) (long-xml-for-topic topic topics-fal state))
 
        (from-xml (str::rchars-to-string
                   (simple-html-encode-chars
@@ -349,6 +363,9 @@
        (acc (json-encode-string from-xml acc))
        (acc (cons #\, acc))
 
+       (acc (json-encode-string (symbol-package-name base-pkg) acc))
+       (acc (cons #\, acc))
+
        (acc (json-encode-string long-str acc))
        (acc (cons #\] acc)))
     (mv acc state)))
@@ -394,11 +411,16 @@
                   (clean-topics topics)))))
 
        (- (cw "; Saving JSON files for ~x0 topics.~%" (len topics)))
-       ((mv topics state)
+       ((mv topics xtopics state)
         (time$ (order-topics-by-importance topics state)
                :msg "; Importance sorting topics: ~st sec, ~sa bytes.~%"
                :mintime 1/2))
 
+       (lcfile (oslib::catpath dir "linkcheck.html"))
+       ((mv channel state) (open-output-channel lcfile :character state))
+       (state (princ$ (linkcheck xtopics) channel state))
+       (state (close-output-channel channel state))
+
        (topics-fal (time$ (topics-fal topics)))
        (uid-map    (time$ (make-uid-map 0 topics nil)))
 
@@ -460,6 +482,7 @@
        (- (cw "Copying fancy viewer main files...~%"))
        (state          (stupid-copy-files xdoc/fancy
                                           (list "collapse_subtopics.png"
+                                                "download.png"
                                                 "expand_subtopics.png"
                                                 "favicon.png"
                                                 "Icon_External_Link.png"
@@ -470,15 +493,20 @@
                                                 "LICENSE"
                                                 "minus.png"
                                                 "plus.png"
+                                                "print.css"
+                                                "print.html"
+                                                "printer.png"
                                                 "render.js"
                                                 "style.css"
                                                 "view_flat.png"
                                                 "view_tree.png"
                                                 "config.js"
                                                 "xdoc.js"
+                                                "xslt.js"
                                                 "xdoc_index.js"
                                                 "xdataget.pl"
                                                 "xdata2sql.pl"
+                                                "zip.sh"
                                                 )
                                           dir state))
 
@@ -503,8 +531,21 @@
     state))
 
 
+(defun run-fancy-zip (dir state)
+  (b* ((- (cw "; XDOC: Running zip.sh to create download/ directory.~%"))
+       ((mv erp val state)
+        (time$ (sys-call+ "sh" (list (oslib::catpath dir "zip.sh") dir) state)
+               :msg "; XDOC zip.sh: ~st sec, ~sa bytes.~%"))
+       ((when erp)
+        (er hard? 'run-fancy-zip
+            "zip.sh failed (exit code ~x0).  ~x1."
+            erp val)
+        state))
+    state))
+
 (defun save-fancy (all-topics dir state)
   (b* ((state (prepare-fancy-dir dir state))
-       (state (save-json-files all-topics dir state)))
+       (state (save-json-files all-topics dir state))
+       (state (run-fancy-zip dir state)))
     state))
 
